/*
 * 该部分服务属于公共服务。
 * 基于基础服务，建立api相关调用。
 */

// 服务
angular.module("starter.services").factory("apiService", [
  "$http",
  "apiConstants",
  "crossSiteService",
  function ($http, apiConstants, crossSiteService) {
    // -----------------------------------------------------------------------------------------------------------------------------

    // 查询豆瓣评论
    function getCommentsFromDoban(isbn, callback) {
      var url =
        apiConstants.bookInteraction.comment.getCommentsFromDoban +
        "/" +
        isbn +
        "?";
      crossSiteService.get(url, callback);
    }

    // -----------------------------------------------------------------------------------------------------------------------------

    // 获取图书详情
    function retrieveBookDetail(bookGroupID, accountId, callback) {
      var url =
        apiConstants.bookInteraction.search.search_id +
        "/" +
        bookGroupID +
        "?accountId=" +
        accountId;
      crossSiteService.get(url, callback);
    }

    // 获取图书分类
    function retrieveBookCategories(callback) {
      var url = apiConstants.bookInteraction.category.enabledCategory + "?";
      crossSiteService.get(url, callback);
    }

    // 获取更多推荐新书
    function retrieveNewRecommendedBooks(accountId, page, pageSize, callback) {
      var url =
        apiConstants.bookInteraction.recommenation.newPageableRecommendedBooks +
        "?";
      url =
        url + "accountId=" + accountId + "&page=" + page + "&size=" + pageSize;
      url = url + "&sortedColumn=createTime";
      crossSiteService.get(url, callback);
    }

    // 获取更多推荐好书
    function retrieveMostRecommendedBooks(accountId, page, pageSize, callback) {
      var url =
        apiConstants.bookInteraction.recommenation
          .mostPageableRecommendedBooks + "?";
      url =
        url + "accountId=" + accountId + "&page=" + page + "&size=" + pageSize;
      crossSiteService.get(url, callback);
    }

    // 查询图书：第一次
    /*
     * wheres: 查询条件对象，条件为“key==value"
     */
    function searchBooks(wheres, id, callback) {
      var url = apiConstants.bookInteraction.search.search + "?";
      if (wheres) {
        var hasArgs = false;
        if (wheres.wrd) {
          url =
            url +
            (hasArgs ? "&" : "") +
            "wrd=" +
            wheres.wrd +
            "&accountId=" +
            id;
          hasArgs = true;
        }
        if (wheres.clc) {
          url =
            url +
            (hasArgs ? "&" : "") +
            "clc=" +
            wheres.clc +
            "&accountId=" +
            id;
          hasArgs = true;
        }
      }
      crossSiteService.get(url, callback);
    }
    // 查询图书：从第二次开始
    function searchBooksMore(set_number, start, end, id, callback) {
      if (set_number && start && end) {
        var url = apiConstants.bookInteraction.search.search + "?";
        url = url + "set_number=" + set_number;
        url = url + "&start=" + start + "&end=" + end + "&accountId=" + id;
        crossSiteService.get(url, callback);
      } else {
        console.warn("searchBooksMore. given argument is not enough");
      }
    }
    // 根据分类码查询图书
    function searchBooksBycategory(argObj, callback) {
      var url = apiConstants.bookInteraction.search.searchBooksBycategory + "?";
      angular.forEach(
        argObj,
        function (value, key) {
          url = url + key + "=" + value + "&";
        },
        url
      );
      crossSiteService.get(url, callback);
    }
    // 获取本地缓存借阅排行榜列表
    function retrieveLocalTopLoanedBooks(argObj, callback) {
      // argObj是参数对象，直接使用键值对表示即可。
      var url =
        apiConstants.bookInteraction.recommenation.localTopLoanedBooks + "?";
      angular.forEach(
        argObj,
        function (value, key) {
          url = url + key + "=" + value + "&";
        },
        url
      );
      crossSiteService.get(url, callback);
    }
    //预约
    function reserveBook(data, callback) {
      var url = apiConstants.bookInteraction.reserveBook;
      crossSiteService.post(url, data, callback);
    }
    //获取预约图书
    function getReserveBook(page, size, borrowId, callback) {
      var url =
        apiConstants.bookInteraction.getReserveBook +
        "/" +
        borrowId +
        "?page=" +
        page +
        "&size=" +
        size;
      crossSiteService.get(url, callback);
    }
    //历史预约
    function historyReserve(page, size, borrowId, callback) {
      var url =
        apiConstants.bookInteraction.historyReserve +
        "/" +
        borrowId +
        "?page=" +
        page +
        "&size=" +
        size;
      crossSiteService.get(url, callback);
    }
    //借阅预约图书
    function loanReserveBooks(data, callback) {
      var url = apiConstants.bookInteraction.loanReserveBooks;
      crossSiteService.post(url, data, callback);
    }
    //刪除預約圖書
    function deleteReserve(borrowerId, reserveItemIds, callback) {
      var url =
        apiConstants.bookInteraction.deleteReserve +
        "/" +
        borrowerId +
        "?reserveItemIds=" +
        reserveItemIds;
      crossSiteService.delete0(url, null, callback);
    }
    //添加一本书到选书筐
    function addBookCart(data, callback) {
      var url = apiConstants.bookInteraction.addBookCart;
      crossSiteService.post(url, data, callback);
    }
    //查询一个读者选书篮中所有的书
    function findCartBooks(borrowId, callback) {
      var url = apiConstants.bookInteraction.findCartBooks + "/" + borrowId;
      crossSiteService.get(url, callback);
    }
    //删除选书篮中的多本书
    function deleteCartBooks(borrowId, groupBookIds, callback) {
      var url =
        apiConstants.bookInteraction.deleteCartBooks +
        "/" +
        borrowId +
        "?groupBookIds=" +
        groupBookIds;
      crossSiteService.delete0(url, null, callback);
    }
    //借书
    function borrowBook(bor_id, bar_code, callback) {
      var url = apiConstants.bookInteraction.borrowBook + "?";
      url = url + "bor_id=" + bor_id + "&bar_code=" + bar_code;
      crossSiteService.get(url, callback);
    }

    // 续借
    function renewBook(bar_code, bor_id, callback) {
      var url = apiConstants.bookInteraction.renewBook + "?";
      url = url + "bar_code=" + bar_code + "&" + "bor_id=" + bor_id;
      crossSiteService.get(url, callback);
    }

    // 还书
    function returnBook(bar_code, callback) {
      var url = apiConstants.bookInteraction.returnBook + "?";
      url = url + "bar_code=" + bar_code;
      crossSiteService.get(url, callback);
    }
    // 查看图书是否可还状态
    function checkReturnOrderAllowed(borrowerId, barCodes, callback) {
      var url = apiConstants.bookInteraction.returnOrderAllowed + "?";
      url = url + "borrowerId=" + borrowerId + "&" + "barCodes=" + barCodes;
      crossSiteService.get(url, callback);
    }
    //查看还书的默认地址和手机号
    function checkReturnDefault(callback) {
      var url = apiConstants.libraryService.checkReturnDefault;
      crossSiteService.get(url, callback);
    }

    // -------------------------------------------------------------------------------------------------------------------------------
    // 生成还书订单(Post)
    function createReturnBookOrder(order, callback) {
      var url = apiConstants.bookInteraction.createReturnBookOrder;
      var data = order;
      crossSiteService.post(url, data, callback);
    }
    // 根据用户id和订单状态分页查询订单信息(Get)
    function queryReturnBookOrdersByBorrowIdAndStatus(
      borrower_id,
      status,
      page,
      pageSize,
      direction,
      callback
    ) {
      var url =
        apiConstants.bookInteraction.queryReturnBookOrdersByBorrowIdAndStatus +
        "?borrowerId=" +
        borrower_id +
        "&orderStatus=" +
        status +
        "&page=" +
        page +
        "&size=" +
        pageSize +
        "&direction=" +
        direction;
      crossSiteService.get(url, callback);
    }
    // 根据barcode查询订单信息(Get)
    // queryReturnBookOrderByBarcode: baseServerUrl + '/bookInteraction' + '/return_orders_barcode',
    function queryNewestReturnBookOrderByBarcode(
      borrower_id,
      barcode,
      callback
    ) {
      var url =
        apiConstants.bookInteraction.queryNewestReturnBookOrderByBarcode;
      url =
        url + "?" + "borrower_id=" + borrower_id + "&" + "barcode=" + barcode;
      crossSiteService.get(url, callback);
    }
    //查看一个用户的所用还书订单
    function checkReturnOrders(borrower_id, callback) {
      var url = apiConstants.bookInteraction.checkReturnOrders;
      url = url + "?borrowerId=" + borrower_id;
      crossSiteService.get(url, callback);
    }
    // -------------------------------------------------------------------------------------------------------------------------------

    // 当前借阅/预约记录
    function retrieveBorrowInfo(bor_id, callback) {
      var url = apiConstants.bookInteraction.borrowInfo;
      url = url + "/" + bor_id;
      crossSiteService.get(url, callback);
    }
    // 历史借阅记录
    function retrieveLoadHistory(bor_id, callback) {
      var url = apiConstants.bookInteraction.loanHistory;
      url = url + "/" + bor_id;
      crossSiteService.get(url, callback);
    }

    // -----------------------------------------------------------------------------------------------------------------------------
    // 活动报名 activity

    // 所有活动（get）
    function retrieveActivities(callback) {
      var url = apiConstants.libraryService.activity.activities;
      crossSiteService.get(url, callback);
    }
    // 通过审核的所有活动（get）
    function retrieveApprovedStatusActivities(page, size, callback) {
      var url =
        apiConstants.libraryService.activity.approved_status_activities +
        "?approved=1" +
        "&page=" +
        page +
        "&size=" +
        size;
      crossSiteService.get(url, callback);
    }
    // 用户创建的活动（get）
    function retrieveOwnerActivities(page, size, user_id, callback) {
      var url = apiConstants.libraryService.activity.owner_activitie;
      url = url + "/" + user_id + "?page=" + page + "&size=" + size;
      crossSiteService.get(url, callback);
    }
    // 创建活动（post）
    function createActivity(activity, callback) {
      var url = apiConstants.libraryService.activity.activity;
      crossSiteService.post(url, activity, callback);
    }
    // 更新活动（Patch）
    function patchActivity(activity, callback) {
      var url = apiConstants.libraryService.activity.delActivity;
      crossSiteService.patch(url, activity, callback);
    }
    // 删除活动（delete）
    function deleteActivity(id, callback) {
      var url = apiConstants.libraryService.activity.delActivity;
      url = url + "/" + id;
      crossSiteService.delete0(url, null, callback);
    }
    // 用户参与的活动（get）
    function retrieveEnrollActivities(
      page,
      size,
      sortedColumn,
      user_id,
      callback
    ) {
      var url = apiConstants.libraryService.activity.enroll_activities;
      url =
        url +
        "/" +
        user_id +
        "?page=" +
        page +
        "&size=" +
        size +
        "&sortedColumn=" +
        sortedColumn;
      crossSiteService.get(url, callback);
    }
    // 报名参加活动（post）
    function enrollActivitiy(data, callback) {
      var url = apiConstants.libraryService.activity.enroll_activity;
      crossSiteService.post(url, data, callback);
    }
    // 取消报名（delete）
    function unenrollActivitiy(activityId, callback) {
      var url = apiConstants.libraryService.activity.enroll_activity;
      url = url + "/" + activityId;
      crossSiteService.delete0(url, null, callback);
    }
    // 根据用户id和活动id查询报名情况
    function queryEnrollStatusByUserId(activityId, userId, callback) {
      var url = apiConstants.libraryService.activity.queryEnrollStatusByUserId;
      url = url + "/" + userId + "?" + "activityId=" + activityId;
      crossSiteService.get(url, callback);
    }

    // -----------------------------------------------------------------------------------------------------------------------------

    // 你选书我买单

    // 获取所有推荐记录（get）
    function retrieveBookChoosingsAll(callback) {
      var url = apiConstants.libraryService.book_choosing.all;
      crossSiteService.get(url, callback);
    }
    // 查询某个用户的推荐记录(GET)
    function retrieveUserBookChoosings(user_id, callback) {
      var url = apiConstants.libraryService.book_choosing.forUser;
      url = url + "/" + user_id;
      crossSiteService.get(url, callback);
    }
    // 新增记录(POST)
    function newBookChoosing(bookChoosing, callback) {
      var url = apiConstants.libraryService.book_choosing.new;
      crossSiteService.post(url, bookChoosing, callback);
    }
    // 根据记录id删除推荐记录(DELETE)
    function deleteBookChoosing(id, callback) {
      var url = apiConstants.libraryService.book_choosing.delete;
      url = url + "/" + id;
      crossSiteService.delete0(url, null, callback);
    }
    // 更新一条记录(PUT)
    function updateBookChoosing(recommendation, callback) {
      var url = apiConstants.libraryService.book_choosing.update;
      url = url + "/" + recommendation;
      crossSiteService.put(url, recommendation, callback);
    }
    // GET 查询某本书被推荐的次数
    function queryBookChoosingCount(isbn, callback) {
      var url =
        apiConstants.libraryService.book_choosing.queryBookChoosingCount;
      url = url + "?" + "isbn=" + isbn;
      crossSiteService.get(url, callback);
    }

    // -----------------------------------------------------------------------------------------------------------------------------
    // 书房/我的书房
    // 查询用户书房信息
    function queryLibraryRoomInfo(userId, callback) {
      var url = apiConstants.libraryService.library_room.queryLibraryRoomInfo;
      url = url + "/" + userId;
      crossSiteService.get(url, callback);
    }
    // 新增一个用户书房信息
    function addLibraryRoomInfo(library_room, callback) {
      var url = apiConstants.libraryService.library_room.addLibraryRoomInfo;
      crossSiteService.post(url, library_room, callback);
    }
    // 修改某个用户书房信息
    function updateLibraryRoomInfo(library_room, callback) {
      var url = apiConstants.libraryService.library_room.updateLibraryRoomInfo;
      crossSiteService.put(url, library_room, callback);
    }
    // 所有图书
    function retrieveAllBooksFromUserLibraryRoom(userId, callback) {
      var url = apiConstants.libraryService.library_room.userBooks;
      url = url + "?" + "userId=" + userId;
      crossSiteService.get(url, callback);
    }
    // 可借图书
    function retrieveLoanableBooksFromUserLibraryRoom(userId, callback) {
      var url = apiConstants.libraryService.library_room.userBooks;
      url = url + "?" + "userId=" + userId + "&" + "&loanable=1";
      crossSiteService.get(url, callback);
    }
    function retrieveDoubanBookInfo(isbn, callback) {
      var url = apiConstants.libraryService.library_room.doubanBookInfo;
      url = url + "/" + isbn;
      crossSiteService.get(url, callback);
    }
    function addBookToLibraryRoom(book, callback) {
      var url = apiConstants.libraryService.library_room.addBookToLibraryRoom;
      crossSiteService.post(url, book, callback);
    }
    function updateBookInLibraryRoom(book, callback) {
      var url =
        apiConstants.libraryService.library_room.updateBookInLibraryRoom;
      crossSiteService.put(url, book, callback);
    }
    function deleteBookFromLibraryRoom(id, callback) {
      var url =
        apiConstants.libraryService.library_room.deleteBookFromLibraryRoom;
      url = url + "/" + id;
      crossSiteService.delete0(url, null, callback);
    }

    // -----------------------------------------------------------------------------------------------------------------------------
    // 发现 周围的人/书房
    function retrieveNearbyPersons(user_id, callback) {
      var url = apiConstants.libraryService.discovery.nearby_persons;
      url = url + "/" + user_id;
      crossSiteService.get(url, callback);
    }

    // -----------------------------------------------------------------------------------------------------------------------------
    // 附近的人
    // POST 上传地理位置
    function uploadPersonLocation(peopleLocation, callback) {
      var url = apiConstants.libraryService.nearby_people.uploadPersonLocation;
      crossSiteService.post(url, peopleLocation, callback);
    }
    // PATCH 更新地理位置
    function updatePersonLocation(peopleLocation, callback) {
      var url = apiConstants.libraryService.nearby_people.updatePersonLocation;
      crossSiteService.patch(url, peopleLocation, callback);
    }
    // GET 获取用户指定半径以内的n条记录
    function getNearbyPeople(userId, radius, count, callback) {
      var url = apiConstants.libraryService.nearby_people.getNearbyPeople;
      url =
        url + "/" + userId + "?" + "radius=" + radius + "&" + "count=" + count;
      crossSiteService.get(url, callback);
    }
    // GET 查询周围用户书房信息
    function getNearbyPeopleLibrooms(userId, radius, count, callback) {
      var url =
        apiConstants.libraryService.nearby_people.getNearbyPeopleLibrooms;
      url =
        url + "/" + userId + "?" + "radius=" + radius + "&" + "count=" + count;
      crossSiteService.get(url, callback);
    }
    // GET 查询其他用户书房的最新更新的图书
    function getRecentBooks(userId, count, callback) {
      var url = apiConstants.libraryService.nearby_people.getRecentBooks;
      url = url + "?" + "userId=" + userId + "&" + "count=" + count;
      crossSiteService.get(url, callback);
    }

    // -----------------------------------------------------------------------------------------------------------------------------
    // misc_query 杂项查询
    // GET 查询某本书是否被用户推荐,是否被用户收藏
    function queryBookConcern(userId, isbn, callback) {
      var url = apiConstants.libraryService.misc_query.queryBookConcern;
      url = url + "/" + userId + "?" + "isbn=" + isbn;
      crossSiteService.get(url, callback);
    }

    // -----------------------------------------------------------------------------------------------------------------------------
    // 你选书我买单-线下模式
    // 获取所有合作书店信息(GET)
    function retrieveAllBookStores(callback) {
      var url = apiConstants.libraryService.bookstore.allStores;
      crossSiteService.get(url, callback);
    }
    // 获取一个合作书店信息(GET)
    function retrieveBookStore(bookstoreId, callback) {
      var url = apiConstants.libraryService.bookstore.store;
      url = url + "/" + bookstoreId;
      crossSiteService.get(url, callback);
    }
    // 查询合作书店里的书籍(GET)
    function retrieveBookStoreBooks(bookstoreId, callback) {
      var url = apiConstants.libraryService.bookstore.book;
      url = url + "/" + bookstoreId;
      crossSiteService.get(url, callback);
    }

    // -----------------------------------------------------------------------------------------------------------------------------
    // 粉丝
    // 查询用户粉丝列表 (GET)
    function queryFans(userId, callback) {
      var url = apiConstants.libraryService.fans.query;
      url = url + "/" + userId;
      crossSiteService.get(url, callback);
    }
    // 取消一位粉丝 (DELETE)
    function removeFan(id, callback) {
      var url = apiConstants.libraryService.fans.remove;
      url = url + "/" + id;
      crossSiteService.delete0(url, null, callback);
    }
    // -----------------------------------------------------------------------------------------------------------------------------
    // 关注
    // 查询用户粉丝列表 (GET)
    function queryFollows(userId, callback) {
      var url = apiConstants.libraryService.follow.query;
      url = url + "/" + userId;
      crossSiteService.get(url, callback);
    }
    // 查询是否关注了某用户 (GET)
    function checkFollow(userId, fansUserId, callback) {
      var url = apiConstants.libraryService.follow.check;
      url = url + "/" + userId + "?fansUserId=" + fansUserId;
      crossSiteService.get(url, callback);
    }
    // 增加一位关注用户 (POST)
    function addFollow(follow, callback) {
      var url = apiConstants.libraryService.follow.add;
      crossSiteService.post(url, follow, callback);
    }
    // 取消一位关注用户 (DELETE)
    function removeFollow(id, callback) {
      var url = apiConstants.libraryService.follow.remove;
      url = url + "/" + id;
      crossSiteService.delete0(url, null, callback);
    }

    // -----------------------------------------------------------------------------------------------------------------------------
    // 图书收藏
    // GET 查询某用户所有收藏
    function queryAllFavorites(userId, callback) {
      var url = apiConstants.libraryService.favorite.queryAllFavorites;
      url = url + "/" + userId;
      crossSiteService.get(url, callback);
    }
    // POST 新增一本收藏
    function addFavorite(favorite, callback) {
      var url = apiConstants.libraryService.favorite.addFavorite;
      crossSiteService.post(url, favorite, callback);
    }
    // DELETE 根据数据id删除收藏
    function removeFavoriteById(id, callback) {
      var url = apiConstants.libraryService.favorite.removeFavoriteById;
      url = url + "/" + id;
      crossSiteService.delete0(url, null, callback);
    }

    // -----------------------------------------------------------------------------------------------------------------------------

    function createOrder(order, callback) {
      crossSiteService.post(
        apiConstants.bookInteraction.order.createOrder,
        order,
        callback
      );
    }
    function findOrderDetail(orderId, callback) {
      var url =
        apiConstants.bookInteraction.order.findOrderDetail + "/" + orderId;
      crossSiteService.get(url, callback);
    }
    function findOrderReturnDetail(orderId, callback) {
      var url =
        apiConstants.bookInteraction.order.findOrderReturnDetail +
        "/" +
        orderId;
      crossSiteService.get(url, callback);
    }
    function getLibraryBranch(callback) {
      var url = apiConstants.libraryService.getLibraryBranch;
      crossSiteService.get(url, callback);
    }

    function payOrder(orderId, callback) {
      var url = apiConstants.bookInteraction.order.payOrder + orderId;
      crossSiteService.patch(url, null, callback);
    }
    // 获取支付宝APP的支付参数
    function getAliPayInfo(orderId, callback) {
      var url =
        apiConstants.bookInteraction.order.getAliPayInfo + "/" + orderId;
      crossSiteService.get(url, callback);
    }
    // 获取微信APP的支付参数
    function getWxPayInfo(orderId, callback) {
      var url = apiConstants.bookInteraction.order.getWxPayInfo + "/" + orderId;
      crossSiteService.get(url, callback);
    }

    function receiverOrder(orderId, callback) {
      var url = apiConstants.bookInteraction.order.receiveOrder + orderId;
      crossSiteService.patch(url, null, callback);
    }

    function closeOrder(orderId, callback) {
      var url = apiConstants.bookInteraction.order.closeOrder + orderId;
      crossSiteService.patch(url, null, callback);
    }

    function getOrdersByStatusAndBorrower(
      borrower_id,
      status,
      page,
      size,
      direction,
      callback
    ) {
      var url =
        apiConstants.bookInteraction.order.findOrdersByStatusAndBorrower +
        "/" +
        borrower_id +
        "?orderStatus=" +
        status +
        "&page=" +
        page +
        "&size=" +
        size +
        "&direction=" +
        direction;
      crossSiteService.get(url, callback);
    }

    function obtainTraceByExpressId(expressId, callback) {
      var url =
        apiConstants.bookInteraction.order.obtainTraceByExpressId + expressId;
      crossSiteService.get(url, callback);
    }
    //查看用户所有的订单数量
    function checkAllOrder(borrowerId, callback) {
      var url =
        apiConstants.bookInteraction.order.checkAllOrder +
        "?borrowerId=" +
        borrowerId;
      crossSiteService.get(url, callback);
    }

    // 消息 message
    // GET 根据用户id获取用户消息
    function getUserMessage(user_id, callback) {
      var url = apiConstants.libraryService.message.getUserMessage;
      url = url + user_id;
      crossSiteService.get(url, callback);
    }
    // GET 根据用户id获取分页的用户消息
    function getUserPageMessage(
      user_id,
      receiverIsAdmin,
      page,
      size,
      sortedColumn,
      callback
    ) {
      var url = apiConstants.libraryService.message.getUserPageMessage;
      if (receiverIsAdmin != true) {
        receiverIsAdmin = false;
      }
      url =
        url +
        "/" +
        user_id +
        "?" +
        "receiverIsAdmin=" +
        receiverIsAdmin +
        "&" +
        "page=" +
        page +
        "&" +
        "size=" +
        size +
        "&" +
        (sortedColumn ? "sortedColumn=" + sortedColumn : "");
      crossSiteService.get(url, callback);
    }
    // POST 创建用户消息
    function createUserMessage(user_message, callback) {
      var url = apiConstants.libraryService.message.createUserMessage;
      crossSiteService.post(url, user_message, callback);
    }
    // Patch 更新用户消息
    function updateUserMessage(user_message, callback) {
      var url = apiConstants.libraryService.message.updateUserMessage;
      crossSiteService.patch(url, user_message, callback);
    }

    // -----------------------------------------------------------------------------------------------------------------------------
    // 短信发送
    // POST 发送修改密码的验证码
    function sendSmsVerificationCodeForChangePassword(phone, callback) {
      var url =
        apiConstants.libraryService.sms
          .sendSmsVerificationCodeForChangePassword +
        "/" +
        phone;
      crossSiteService.post(url, null, callback);
    }

    // GET 查看验证码是否正确
    function verifySmsVerificationCode(userId, verifyCode, callback) {
      var url = apiConstants.libraryService.sms.verifySmsVerificationCode;
      url = url + "/" + userId + "?verifyCode=" + verifyCode;
      crossSiteService.get(url, callback);
    }

    // -----------------------------------------------------------------------------------------------------------------------------
    // 用户登录
    function userLogin(account, password, callback) {
      var url = apiConstants.bookInteraction.user.login;
      var data = {
        account: account,
        password: password,
      };
      crossSiteService.postForm(url, data, callback);
    }
    // 用户退出
    function userLogout(callback) {
      var url = apiConstants.bookInteraction.user.logout;
      crossSiteService.get(url, callback);
    }
    // GET 获取用户信息
    function getUserInfo(userId, callback) {
      var url = apiConstants.bookInteraction.user.getUserInfo;
      url = url + "/" + userId;
      crossSiteService.get(url, callback);
    }
    //用户可借本数
    function borrowCount(userId, callback) {
      var url = apiConstants.bookInteraction.user.borrowCount;
      url = url + "/" + userId;
      crossSiteService.get(url, callback);
    }
    //新增用户地址
    function addAddress(addressInfo, callback) {
      var url = apiConstants.address.addAddress;
      crossSiteService.post(url, addressInfo, callback);
    }
    //获取一个人的所有地址
    function addressAll(account_id, callback) {
      var url = apiConstants.address.addressAll + "/" + account_id;
      crossSiteService.get(url, callback);
    }
    //更新一个地址
    function updateAddress(updateInfo, callback) {
      var url = apiConstants.address.updateAddress;
      crossSiteService.post(url, updateInfo, callback);
    }
    //删除一个地址
    function deleteAddress(addressId, callback) {
      var url = apiConstants.address.deleteAddress + "/" + addressId;
      crossSiteService.delete0(url, null, callback);
    }
    //设置默认地址
    function defaultAddress(account_id, addressId, callback) {
      var url =
        apiConstants.address.defaultAddress +
        account_id +
        "?addressId=" +
        addressId;
      crossSiteService.put(url, null, callback);
    }
    //查询一个人的默认地址
    function findDefaultAddress(account_id, callback) {
      var url = apiConstants.address.findDefaultAddress + "/" + account_id;
      crossSiteService.get(url, callback);
    }
    //上传设备信息
    function deviceInfo(info, callback) {
      var url = apiConstants.bookInteraction.user.deviceInfo;
      crossSiteService.post(url, info, callback);
    }
    // -----------------------------------------------------------------------------------------------------------------------------
    // 读者信息修改接口
    // POST 修改登录密码
    function changeReaderPassword(pwdInfo, callback) {
      var url = apiConstants.bookInteraction.reader.change_password;
      crossSiteService.post(url, pwdInfo, callback);
    }

    // -----------------------------------------------------------------------------------------------------------------------------
    // ---------------------------------------------------------孩子信息管理---------------------------------------------------------
    // GET  查询某个用户所有儿童信息
    function queryChildrenInfoAll(userId, callback) {
      var url = apiConstants.libraryService.children.queryChildrenInfoAll;
      url += "/" + userId;
      crossSiteService.get(url, callback);
    }
    // POST 添加一个儿童信息
    function addChildInfo(child, callback) {
      var url = apiConstants.libraryService.children.addChildInfo;
      crossSiteService.post(url, child, callback);
    }
    // PUT  修改一个儿童信息
    function updateChildInfo(child, callback) {
      var url = apiConstants.libraryService.children.updateChildInfo;
      crossSiteService.put(url, child, callback);
    }
    // DELETE 根据id删除用户所有儿童信息
    function deleteChildInfo(id, callback) {
      var url = apiConstants.libraryService.children.deleteChildInfo;
      url += "/" + id;
      crossSiteService.delete0(url, null, callback);
    }

    //---------------------------------------------注册电子证--------------------------------------------------------//
    function getVcodeSending(telePhone, callback) {
      var url = apiConstants.bookInteraction.register.getVcodeSending;
      url += "/" + telePhone;
      crossSiteService.get(url, callback);
    }
    function checkVcodeSending(telePhone, code, callback) {
      var url = apiConstants.bookInteraction.register.checkVcodeSending;
      url += "/" + telePhone + "?vcode=" + code;
      crossSiteService.get(url, callback);
    }
    function getCardInfo(idCardNum, callback) {
      var url = apiConstants.bookInteraction.register.getCardInfo;
      url += "/" + idCardNum;
      crossSiteService.get(url, callback);
    }
    function registerCard(registerInfo, callback) {
      var url = apiConstants.bookInteraction.register.registerCard;
      crossSiteService.post(url, registerInfo, callback);
    }
    function cashAlipay(idCardNum, callback) {
      var url = apiConstants.bookInteraction.register.cashAlipay;
      url += "/" + idCardNum;
      crossSiteService.get(url, callback);
    }
    function cashWxpay(idCardNum, callback) {
      var url = apiConstants.bookInteraction.register.cashWxpay;
      url += "/" + idCardNum;
      crossSiteService.get(url, callback);
    }
    function updateCashInAleph(orderId, callback) {
      var url = apiConstants.bookInteraction.register.updateCashInAleph;
      url += "/" + orderId;
      crossSiteService.get(url, callback);
    }

    //--------------------------------------------扫码开门--------------------------------------------//
    function scanBarcodeOpen(userId, userName, qrcode, callback) {
      var url = apiConstants.bookRoom.scanBarcode.scanBarcodeOpen;
      url += "?userId=" + userId + "&userName=" + userName + "&" + qrcode;
      crossSiteService.get(url, callback);
    }

    //--------------------------------------------海恒设备登录---------------------------------------------//
    function haihengScanQRcode(userId, params, callback) {
      var url = apiConstants.bookRoom.haihengDeviceScanQRcode.scanQRcodeByApp;
      url += "?userId=" + userId + "&" + params;
      crossSiteService.get(url, callback);
    }

    //人脸
    function queryFacePic(account, callback) {
      var url = apiConstants.bookRoom.faceRecognition.queryBindFace;
      url += "/" + account;
      crossSiteService.get(url, callback);
    }

    function unboundUserFace(account, callback) {
      var url = apiConstants.bookRoom.faceRecognition.unboundFace;
      url += "/" + account;
      crossSiteService.delete0(url, null, callback);
    }

    function gedFaceBindStatus(account, callback) {
      var url = apiConstants.bookRoom.faceRecognition.judgeBindStatus;
      url += "/" + account;
      crossSiteService.get(url, callback);
    }

    //-------------------------------------------登录进入电子资源---------------------------------------------//
    function electronicRead(userId, callback) {
      var url = apiConstants.bookInteraction.electronicRead;
      url += "/" + userId;
      crossSiteService.get(url, callback);
    }

    function onlineBookstore(userId, callback) {
      var url = apiConstants.bookInteraction.onlineBookstore;
      url += "/" + userId;
      crossSiteService.get(url, callback);
    }
    //进入电子期刊
    function electronicJournal(userId, callback) {
      var url = apiConstants.bookRoom.electronicJournal;
      url += "/" + userId;
      crossSiteService.get(url, callback);
    }
    //进入龙源音频
    function longyuanAudio(userId, callback) {
      var url = apiConstants.bookRoom.longyuanAudio;
      url += "/" + userId;
      crossSiteService.get(url, callback);
    }
    // -----------------------------------------------------------------------------------------------------------------------------
    // GET 获取最新的版本信息
    var retrieveNewestUpgradeVersion = function (appInfo, callback) {
      var url = apiConstants.libraryService.app.retrieveNewestUpgradeVersion;
      url = url + "?";
      if (appInfo) {
        angular.forEach(appInfo, function (value, key) {
          url = url + "&" + key + "=" + value;
        });
      }
      crossSiteService.get(url, callback);
    };

    // -----------------------------------------------------------------------------------------------------------------------------
    // 首页数据
    // GET 获取用户信息
    function getHomePageSlideImages(callback) {
      var url = apiConstants.libraryService.homePage.slideImages;
      crossSiteService.get(url, callback);
    }

    // -----------------------------------------------------------------------------------------------------------------------------
    return {
      // comment
      getCommentsFromDoban: getCommentsFromDoban,
      // book public
      retrieveBookDetail: retrieveBookDetail,
      retrieveBookCategories: retrieveBookCategories,
      retrieveNewRecommendedBooks: retrieveNewRecommendedBooks,
      retrieveMostRecommendedBooks: retrieveMostRecommendedBooks,
      searchBooks: searchBooks,
      searchBooksMore: searchBooksMore,
      searchBooksBycategory: searchBooksBycategory,
      retrieveLocalTopLoanedBooks: retrieveLocalTopLoanedBooks,
      // book private
      addBookCart: addBookCart,
      findCartBooks: findCartBooks,
      deleteCartBooks: deleteCartBooks,
      borrowBook: borrowBook,
      renewBook: renewBook,
      returnBook: returnBook,
      reserveBook: reserveBook,
      getReserveBook: getReserveBook,
      historyReserve: historyReserve,
      loanReserveBooks: loanReserveBooks,
      deleteReserve: deleteReserve,
      checkReturnOrderAllowed: checkReturnOrderAllowed,
      retrieveBorrowInfo: retrieveBorrowInfo,
      retrieveLoadHistory: retrieveLoadHistory,
      checkReturnDefault: checkReturnDefault,
      // 还书订单物流
      createReturnBookOrder: createReturnBookOrder, // 生成还书订单(Post)
      queryReturnBookOrdersByBorrowIdAndStatus: queryReturnBookOrdersByBorrowIdAndStatus, // 根据用户id和订单状态分页查询订单信息(Get)
      queryNewestReturnBookOrderByBarcode: queryNewestReturnBookOrderByBarcode, // 根据barcode查询订单信息(Get)
      checkReturnOrders: checkReturnOrders,
      // activity public
      retrieveActivities: retrieveActivities,
      retrieveApprovedStatusActivities: retrieveApprovedStatusActivities,
      // activity private
      retrieveOwnerActivities: retrieveOwnerActivities,
      createActivity: createActivity,
      patchActivity: patchActivity,
      deleteActivity: deleteActivity,
      retrieveEnrollActivities: retrieveEnrollActivities,
      enrollActivitiy: enrollActivitiy,
      unenrollActivitiy: unenrollActivitiy,
      queryEnrollStatusByUserId: queryEnrollStatusByUserId,
      // book_choosing
      retrieveBookChoosingsAll: retrieveBookChoosingsAll,
      retrieveUserBookChoosings: retrieveUserBookChoosings,
      newBookChoosing: newBookChoosing,
      deleteBookChoosing: deleteBookChoosing,
      updateBookChoosing: updateBookChoosing,
      queryBookChoosingCount: queryBookChoosingCount,
      // book store
      retrieveAllBookStores: retrieveAllBookStores,
      retrieveBookStore: retrieveBookStore,
      retrieveBookStoreBooks: retrieveBookStoreBooks,
      // library_room
      queryLibraryRoomInfo: queryLibraryRoomInfo,
      addLibraryRoomInfo: addLibraryRoomInfo,
      updateLibraryRoomInfo: updateLibraryRoomInfo,
      retrieveAllBooksFromUserLibraryRoom: retrieveAllBooksFromUserLibraryRoom,
      retrieveLoanableBooksFromUserLibraryRoom: retrieveLoanableBooksFromUserLibraryRoom,
      retrieveDoubanBookInfo: retrieveDoubanBookInfo,
      retrieveUserBookChoosings: retrieveUserBookChoosings,
      addBookToLibraryRoom: addBookToLibraryRoom,
      updateBookInLibraryRoom: updateBookInLibraryRoom,
      deleteBookFromLibraryRoom: deleteBookFromLibraryRoom,
      // discovery
      retrieveNearbyPersons: retrieveNearbyPersons,
      // nearby
      uploadPersonLocation: uploadPersonLocation,
      updatePersonLocation: updatePersonLocation,
      getNearbyPeople: getNearbyPeople,
      getNearbyPeopleLibrooms: getNearbyPeopleLibrooms,
      getRecentBooks: getRecentBooks,
      //order
      createOrder: createOrder,
      getLibraryBranch: getLibraryBranch,
      payOrder: payOrder,
      getAliPayInfo: getAliPayInfo,
      getWxPayInfo: getWxPayInfo,
      receiverOrder: receiverOrder,
      closeOrder: closeOrder,
      getOrdersByStatusAndBorrower: getOrdersByStatusAndBorrower,
      obtainTraceByExpressId: obtainTraceByExpressId,
      findOrderDetail: findOrderDetail,
      findOrderReturnDetail: findOrderReturnDetail,
      checkAllOrder: checkAllOrder,
      // fans
      queryFans: queryFans,
      removeFan: removeFan,
      // follows
      queryFollows: queryFollows,
      checkFollow: checkFollow,
      addFollow: addFollow,
      removeFollow: removeFollow,
      // favorite
      queryAllFavorites: queryAllFavorites,
      addFavorite: addFavorite,
      removeFavoriteById: removeFavoriteById,
      // misc_query
      queryBookConcern: queryBookConcern,
      // message
      getUserMessage: getUserMessage,
      getUserPageMessage: getUserPageMessage,
      createUserMessage: createUserMessage,
      updateUserMessage: updateUserMessage,
      // sms
      sendSmsVerificationCodeForChangePassword: sendSmsVerificationCodeForChangePassword,
      verifySmsVerificationCode: verifySmsVerificationCode,
      // user private
      userLogin: userLogin,
      userLogout: userLogout,
      getUserInfo: getUserInfo,
      borrowCount: borrowCount,
      //address
      addAddress: addAddress,
      addressAll: addressAll,
      updateAddress: updateAddress,
      deleteAddress: deleteAddress,
      defaultAddress: defaultAddress,
      findDefaultAddress: findDefaultAddress,
      // reader
      changeReaderPassword: changeReaderPassword,
      // children
      queryChildrenInfoAll: queryChildrenInfoAll,
      addChildInfo: addChildInfo,
      updateChildInfo: updateChildInfo,
      deleteChildInfo: deleteChildInfo,
      //register
      getVcodeSending: getVcodeSending,
      checkVcodeSending: checkVcodeSending,
      getCardInfo: getCardInfo,
      registerCard: registerCard,
      cashAlipay: cashAlipay,
      cashWxpay: cashWxpay,
      updateCashInAleph: updateCashInAleph,
      //扫码开门
      scanBarcodeOpen: scanBarcodeOpen,
      //海恒设备登录
      haihengScanQRcode: haihengScanQRcode,
      queryFacePic: queryFacePic,
      unboundUserFace: unboundUserFace,
      gedFaceBindStatus: gedFaceBindStatus,
      // app
      retrieveNewestUpgradeVersion: retrieveNewestUpgradeVersion,
      // home page
      getHomePageSlideImages: getHomePageSlideImages,
      deviceInfo,
      //chaoxing
      electronicRead: electronicRead,
      onlineBookstore: onlineBookstore,
      //电子期刊
      electronicJournal: electronicJournal,
      //电子音频
      longyuanAudio: longyuanAudio,
    };
  },
]);
