/*
 * 该部分服务属于公共服务。
 * 该部分服务是一些自定的基础服务。
 * 主要是一些通用的Log调用。
 */

// Log日志服务
angular.module("starter.services").factory("appLogService", [
  "$rootScope",
  "logConstants",
  "crossSiteService",
  "cordovaService",
  "ngCordovaService",
  function (
    $rootScope,
    logConstants,
    crossSiteService,
    cordovaService,
    ngCordovaService
  ) {
    var log_date_templet = {
      // basic 0
      project: undefined,
      // basic 1
      device_info: undefined,
      network_info: undefined,
      // baic 2
      app_info: undefined,
      user_info: undefined,
      time: undefined,
      // special
      level: undefined, // 'log', 'info', 'error', 'debug'
      category: undefined, // 'syntax':语法错误, 'module':模块错误
      module: undefined, // 模块
      error: undefined, // 对于语法错误，则是{msg,url,line}
    };

    function getBaseLog() {
      var log = angular.copy(log_date_templet);
      // basic 0
      log.project = "library_app_ionic";
      // basic 1
      log.device_info = cordovaService.getDeviceInfo();
      log.network_info = cordovaService.getNetworkInfo();
      // basic 2
      log.app_info = getCheckedAppInfo();
      log.user_info = $rootScope.userInfo;
      log.time = new Date().toUTCString();
      //
      return log;
    }

    function uploadLog(log, callback) {
      log = getLogEncapsulation(log);
      var url = logConstants.api.upload;
      console.log("uploadLog. url = " + url);
      crossSiteService.post(url, log, callback);
    }
    function getLogEncapsulation(log) {
      var logEncapsulation = {
        userId: undefined,
        imei: undefined,
        data: undefined,
      };
      if (log instanceof Object) {
        logEncapsulation.userId = $rootScope.userInfo
          ? $rootScope.userInfo.account
          : undefined;
        // logEncapsulation.imei = undefined; // TODO 后面实现
        logEncapsulation.data = angular.toJson(log);
      }
      return logEncapsulation;

      // if (log instanceof Object) {
      //     // basic 0
      //     if (log.project && log.project instanceof Object) {
      //         log.project = angular.toJson(log.project);
      //     }
      //     // basic 1
      //     if (log.device_info && log.device_info instanceof Object) {
      //         log.device_info = angular.toJson(log.device_info);
      //     }
      //     if (log.network_info && log.network_info instanceof Object) {
      //         log.network_info = angular.toJson(log.network_info);
      //     }
      //     // baic 2
      //     if (log.app_info && log.app_info instanceof Object) {
      //         log.app_info = angular.toJson(log.app_info);
      //     }
      //     if (log.user_info && log.user_info instanceof Object) {
      //         log.user_info = angular.toJson(log.user_info);
      //     }
      //     if (log.time && log.time instanceof Object) {
      //         log.time = angular.toJson(log.time);
      //     }
      //     // special
      //     if (log.level && log.level instanceof Object) {
      //         log.level = angular.toJson(log.level);
      //     }
      //     if (log.category && log.category instanceof Object) {
      //         log.category = angular.toJson(log.category);
      //     }
      //     if (log.module && log.module instanceof Object) {
      //         log.module = angular.toJson(log.module);
      //     }
      //     if (log.error && log.error instanceof Object) {
      //         log.error = angular.toJson(log.error);
      //     }
      // }
    }

    // 处理普通错误
    onerror = function (msg, url, line) {
      //
      var txt = "";
      txt = "There was an error on this page.\n\n";
      txt += "Error: " + msg + "\n";
      txt += "URL: " + url + "\n";
      txt += "Line: " + line + "\n\n";
      txt += "Click OK to continue.\n\n";
      console.log("发现普通语法错误: " + txt);
      //
      var log = getBaseLog();
      log.level = "error";
      log.category = "syntax";
      log.error = {
        msg: msg,
        url: url,
        line: line,
      };
      uploadLog(log, {
        onSuccess: function (data, status, config, headers) {
          console.log("提交普通语法错误日志-->成功");
        },
        onError: function (data, status, config, headers) {
          console.error("提交普通语法错误日志-->失败");
        },
        onFinally: function () {
          //
        },
      });
    };

    // --------------------------------------------------------------------------------------------------------------------------------------
    // 这里不能再使用appServices,因为，appServices中也引入了当前的service。不同service之间的依赖不能形成闭环。
    //
    var checkedAppInfo = {}; // 已检测的版本信息
    var getCheckedAppInfo = function () {
      return checkedAppInfo;
    };
    // check app version
    var checkAppInfo = function (callbacks) {
      //
      ngCordovaService.getAppVersionNumber({
        onSuccess: function (versionNumber) {
          console.log("checkAppInfo. versionNumber = " + versionNumber);
          checkedAppInfo.versionNumber = versionNumber;
          if (
            callbacks &&
            callbacks.getAppVersionNumberCallback &&
            callbacks.getAppVersionNumberCallback.onSuccess
          ) {
            callbacks.getAppVersionNumberCallback.onSuccess(versionNumber);
          }
        },
        onError: function (err) {
          if (
            callbacks &&
            callbacks.getAppVersionNumberCallback &&
            callbacks.getAppVersionNumberCallback.onError
          ) {
            callbacks.getAppVersionNumberCallback.onError(err);
          }
        },
      });
      //
      ngCordovaService.getAppVersionCode({
        onSuccess: function (versionCode) {
          console.log("checkAppInfo. versionCode = " + versionCode);
          checkedAppInfo.versionCode = versionCode;
          if (
            callbacks &&
            callbacks.getAppVersionCodeCallback &&
            callbacks.getAppVersionCodeCallback.onSuccess
          ) {
            callbacks.getAppVersionCodeCallback.onSuccess(versionCode);
          }
        },
        onError: function (err) {
          if (
            callbacks &&
            callbacks.getAppVersionCodeCallback &&
            callbacks.getAppVersionCodeCallback.onError
          ) {
            callbacks.getAppVersionCodeCallback.onError(err);
          }
        },
      });
      //
      ngCordovaService.getAppName({
        onSuccess: function (appName) {
          console.log("checkAppInfo. appName = " + appName);
          checkedAppInfo.appName = appName;
          if (
            callbacks &&
            callbacks.getAppNameCallback &&
            callbacks.getAppNameCallback.onSuccess
          ) {
            callbacks.getAppNameCallback.onSuccess(appName);
          }
        },
        onError: function (err) {
          if (
            callbacks &&
            callbacks.getAppNameCallback &&
            callbacks.getAppNameCallback.onError
          ) {
            callbacks.getAppNameCallback.onError(err);
          }
        },
      });
      //
      ngCordovaService.getAppPackageName({
        onSuccess: function (appPackageName) {
          console.log("checkAppInfo. appPackageName = " + appPackageName);
          checkedAppInfo.appPackageName = appPackageName;
          if (
            callbacks &&
            callbacks.getAppPackageNameCallback &&
            callbacks.getAppPackageNameCallback.onSuccess
          ) {
            callbacks.getAppPackageNameCallback.onSuccess(appPackageName);
          }
        },
        onError: function (err) {
          if (
            callbacks &&
            callbacks.getAppPackageNameCallback &&
            callbacks.getAppPackageNameCallback.onError
          ) {
            callbacks.getAppPackageNameCallback.onError(err);
          }
        },
      });
    };
    document.addEventListener("deviceready", checkAppInfo, false);

    // --------------------------------------------------------------------------------------------------------------------------------------
    return {
      getBaseLog: getBaseLog,
      uploadLog: uploadLog,
    };
  },
]);
