/*
 * 该部分服务属于公共服务。
 * 该部分服务主要和版本操作相关。
 */

// UI服务
angular.module("starter.services").factory("appService", [
  "$rootScope",
  "$ionicPopup",
  "$timeout",
  "ngCordovaService",
  "apiService",
  "appLogService",
  function (
    $rootScope,
    $ionicPopup,
    $timeout,
    ngCordovaService,
    apiService,
    appLogService
  ) {
    // --------------------------------------------------------------------------------------------------------------------------------------
    //
    var checkedAppInfo = {}; // 已检测的版本信息
    var getCheckedAppInfo = function () {
      return checkedAppInfo;
    };
    // check app version
    var checkAppInfo = function (callbacks) {
      //
      ngCordovaService.getAppVersionNumber({
        onSuccess: function (versionNumber) {
          console.log("checkAppInfo. versionNumber = " + versionNumber);
          checkedAppInfo.versionNumber = versionNumber;
          if (
            callbacks &&
            callbacks.getAppVersionNumberCallback &&
            callbacks.getAppVersionNumberCallback.onSuccess
          ) {
            callbacks.getAppVersionNumberCallback.onSuccess(versionNumber);
          }
        },
        onError: function (err) {
          if (
            callbacks &&
            callbacks.getAppVersionNumberCallback &&
            callbacks.getAppVersionNumberCallback.onError
          ) {
            callbacks.getAppVersionNumberCallback.onError(err);
          }
        },
      });
      //
      ngCordovaService.getAppVersionCode({
        onSuccess: function (versionCode) {
          console.log("checkAppInfo. versionCode = " + versionCode);
          checkedAppInfo.versionCode = versionCode;
          if (
            callbacks &&
            callbacks.getAppVersionCodeCallback &&
            callbacks.getAppVersionCodeCallback.onSuccess
          ) {
            callbacks.getAppVersionCodeCallback.onSuccess(versionCode);
          }
        },
        onError: function (err) {
          if (
            callbacks &&
            callbacks.getAppVersionCodeCallback &&
            callbacks.getAppVersionCodeCallback.onError
          ) {
            callbacks.getAppVersionCodeCallback.onError(err);
          }
        },
      });
      //
      ngCordovaService.getAppName({
        onSuccess: function (appName) {
          console.log("checkAppInfo. appName = " + appName);
          checkedAppInfo.appName = appName;
          if (
            callbacks &&
            callbacks.getAppNameCallback &&
            callbacks.getAppNameCallback.onSuccess
          ) {
            callbacks.getAppNameCallback.onSuccess(appName);
          }
        },
        onError: function (err) {
          if (
            callbacks &&
            callbacks.getAppNameCallback &&
            callbacks.getAppNameCallback.onError
          ) {
            callbacks.getAppNameCallback.onError(err);
          }
        },
      });
      //
      ngCordovaService.getAppPackageName({
        onSuccess: function (appPackageName) {
          console.log("checkAppInfo. appPackageName = " + appPackageName);
          checkedAppInfo.appPackageName = appPackageName;
          if (
            callbacks &&
            callbacks.getAppPackageNameCallback &&
            callbacks.getAppPackageNameCallback.onSuccess
          ) {
            callbacks.getAppPackageNameCallback.onSuccess(appPackageName);
          }
        },
        onError: function (err) {
          if (
            callbacks &&
            callbacks.getAppPackageNameCallback &&
            callbacks.getAppPackageNameCallback.onError
          ) {
            callbacks.getAppPackageNameCallback.onError(err);
          }
        },
      });
    };

    // --------------------------------------------------------------------------------------------------------------------------------------
    //
    var newestAppVersion = {}; // 最新版本信息
    //
    var getNewestAppVersion = function () {
      return newestAppVersion;
    };
    // 检查（服务器）最新版本信息
    var isRetrievingNewestAppVersion = false;
    var retrieveNewestAppVersion = function (toHandle, silent) {
      //
      if (
        appFileDowningTaskStatus &&
        appFileDowningTaskStatus.status == "running"
      ) {
        showAppDownloadingProgressbar();
        return;
      }
      //
      if (isRetrievingNewestAppVersion) {
        return;
      }
      //
      isRetrievingNewestAppVersion = true;
      // TODO 模拟获取最新版本信息 临时假数据
      var appType = null;
      if (ionic.Platform.isAndroid()) {
        appType = 1;
      } else if (ionic.Platform.isIPad() || ionic.Platform.isIOS()) {
        appType = 2;
      }
      if (appType == null) {
        console.warn("不被支持的平台");
        return;
      }
      var appInfo = { appType: appType };
      apiService.retrieveNewestUpgradeVersion(appInfo, {
        onSuccess: function (data, status, config, headers) {
          newestAppVersion = data ? data : {};
          if (toHandle) {
            handleNewestAppVersion(newestAppVersion, silent);
          }
        },
        onError: function (data, status, config, headers) {
          //
        },
        onFinally: function () {
          isRetrievingNewestAppVersion = false;
        },
      });
    };
    var handleNewestAppVersion = function (newestAppVersion, silent) {
      if (checkedAppInfo.versionCode) {
        if (newestAppVersion.versionCode < checkedAppInfo.versionCode) {
          console.warn("服务器版本比当前版本还要低，请联系服务器管理员。");
        } else if (newestAppVersion.versionCode == checkedAppInfo.versionCode) {
          console.warn("服务器版本与当前版本一致，不需要升级。");
          if (silent === true) {
            return;
          }
          ngCordovaService.showToast("当前已经是最新版本");
        } else if (newestAppVersion.versionCode > checkedAppInfo.versionCode) {
          console.warn("服务器版本比当前版本高，建议升级到该版本。");
          if (newestAppVersion.forceUpdate == true) {
            console.warn("服务器版本要求强制升级。");
            downloadAppFile(newestAppVersion.url);
          } else {
            console.warn("服务器版本不要求强制升级。");
            if (silent != true) {
              ngCordovaService.showToast("当前已经是最新版本");
            } else {
              var message =
                "名称：" +
                newestAppVersion.versionName +
                "\n" +
                "\n" +
                "内容：" +
                "\n" +
                newestAppVersion.brife;
              var title = "升级版本";
              var buttonArray = ["升级", "暂不升级"];
              var buttonCallbackArray = [
                function () {
                  downloadAppFile(newestAppVersion.url);
                },
                function () {},
              ];
              ngCordovaService.showDailogConfirm(
                message,
                title,
                buttonArray,
                buttonCallbackArray
              );
            }
          }
        }
      } else {
        console.warn("请先检查最新版本，再尝试下载APP...");
      }
    };
    //
    var baseAppFileDowningTaskStatus = {
      status: "idel", // 'idel', 'running', 'successful', 'failed'
      progress: {
        total: 0,
        loaded: 0,
        percent: 0, // (loaded / total) + '%'
      },
    };
    var appFileDowningTaskStatus = undefined;
    var downloadAppFile = function (url) {
      // 防止重复下载
      if (
        appFileDowningTaskStatus &&
        appFileDowningTaskStatus.status == "running"
      ) {
        // ngCordovaService.showToast('已经有一个更新任务正在进行 ...');
        showAppDownloadingProgressbar();
        return;
      } else {
        appFileDowningTaskStatus = angular.copy(baseAppFileDowningTaskStatus);
        $rootScope.appFileDowningTaskStatus = appFileDowningTaskStatus;
      }
      //
      var callback = {
        onSuccess: function (result) {
          appFileDowningTaskStatus.status = "successful";
          appFileDowningTaskStatus.progress.percent = 100;
          console.log("正在下载最新版本，已完成" + 100 + "%");
          ngCordovaService.showToast("已经完成下载");
          closeAppDownloadingProgressbar();
          // check md5
          // TODO
          ngCordovaService.showToast("文件校验通过");
          // open app file or install app
          ngCordovaService.showToast("即将安装新的APP ...");
          if (ionic.Platform.isIPad() || ionic.Platform.isIOS()) {
            // IOS
            // TODO
          } else if (ionic.Platform.isAndroid()) {
            // Android
            ngCordovaService.installAndroidApk(result.nativeURL, {
              onSuccess: function () {
                console.log("已经成功打开apk文件。");
              },
              onError: function (err) {
                // err: {message, status,}
                console.warn("无法安装apk，错误信息是：" + err);
                ngCordovaService.showToast("无法安装apk");
                //
                handleAppUpgradeError(err);
              },
            });
          } else {
            // Others are not supported
            ngCordovaService.showToast("暂不支持该平台");
            return;
          }
        },
        onError: function (err) {
          // err: {body, code, exception, http_status, source, target,}
          appFileDowningTaskStatus.status = "error";
          ngCordovaService.showToast("下载过程中出现错误");
          closeAppDownloadingProgressbar();
          console.log(err);
          //
          handleAppUpgradeError(err);
        },
        onProgress: function (progress) {
          appFileDowningTaskStatus.progress.loaded = progress.loaded;
          appFileDowningTaskStatus.progress.total = progress.total;
          var downloadProgressPercent =
            Math.round((progress.loaded / progress.total) * 10000) / 100.0;
          downloadProgressPercent = Math.ceil(
            (progress.loaded / progress.total) * 100
          );
          appFileDowningTaskStatus.progress.percent = downloadProgressPercent;
          console.log(
            "正在下载最新版本，已完成" + downloadProgressPercent + "%"
          );
        },
      };
      //
      var targetPath = null;
      if (ionic.Platform.isIPad() || ionic.Platform.isIOS()) {
        // IOS
        targetPath = cordova.file.documentsDirectory + "app.ipa";
        console.log("downloadAppFile. targetPath = " + targetPath);
        ngCordovaService.removeFile(cordova.file.documentsDirectory, "app.ipa");
      } else if (ionic.Platform.isAndroid()) {
        // Android
        targetPath = cordova.file.externalRootDirectory + "app.apk";
        console.log("downloadAppFile. targetPath = " + targetPath);
        ngCordovaService.checkFile(
          cordova.file.externalRootDirectory,
          "app.apk",
          {
            onSuccess: function (result) {
              // success
              ngCordovaService.removeFile(
                cordova.file.externalRootDirectory,
                "app.apk",
                {
                  onSuccess: function (result) {
                    // success
                    ngCordovaService.showToast("正在下载最新版本...");
                    appFileDowningTaskStatus.status = "running";
                    showAppDownloadingProgressbar();
                    ngCordovaService.download(url, targetPath, callback);
                  },
                  onError: function (err) {
                    // error
                    console.error("删除文件失败。请再次尝试（删除）。。。");
                    handleAppUpgradeError(err);
                  },
                }
              );
            },
            onError: function (err) {
              // error
              console.warn("检查文件失败。");
              ngCordovaService.showToast("正在下载最新版本...");
              appFileDowningTaskStatus.status = "running";
              showAppDownloadingProgressbar();
              ngCordovaService.download(url, targetPath, callback);
            },
          }
        );
      } else {
        // Others are not supported
        ngCordovaService.showToast("暂不支持该平台");
        return;
      }
    };

    var appDownloadingProgressbar = undefined;
    function showAppDownloadingProgressbar() {
      appDownloadingProgressbar = $ionicPopup.show({
        cssClass: "popup-container-upgrade",
        templateUrl: "templates/download_progress.html",
        title: "正在下载更新文件...",
        scope: $rootScope,
        buttons: [
          {
            text: "后台运行",
            type: "button button-balanced",
          },
        ],
      });
    }
    function closeAppDownloadingProgressbar() {
      if (appDownloadingProgressbar) {
        appDownloadingProgressbar.close();
      }
    }

    function handleAppUpgradeError(err) {
      //
      if (
        err &&
        ((err.exception &&
          err.exception.indexOf("(Permission denied)") == -1) ||
          (err.message && err.message.indexOf("(SECURITY_ERR)") == -1))
      ) {
        ngCordovaService.showToast(
          "应用缺少相关权限，请在“应用信息”的“权限管理”中打开“读写手机存储”权限。"
        );
      }
      //
      var log = appLogService.getBaseLog();
      log.level = "error";
      log.category = "module";
      log.module = "app_update";
      log.error = err;
      appLogService.uploadLog(log, {
        onSuccess: function (data, status, config, headers) {
          console.log("提交APP升级错误日志-->成功");
        },
        onError: function (data, status, config, headers) {
          console.error("提交APP升级错误日志-->失败");
        },
        onFinally: function () {
          //
        },
      });
    }

    // --------------------------------------------------------------------------------------------------------------------------------------
    return {
      checkAppInfo: checkAppInfo,
      getCheckedAppInfo: getCheckedAppInfo,
      retrieveNewestAppVersion: retrieveNewestAppVersion,
      getNewestAppVersion: getNewestAppVersion,
    };
  },
]);
