// ngCordova服务
angular.module("starter.services").factory("ngCordovaService", [
  "$cordovaGeolocation",
  "$cordovaSocialSharing",
  "$cordovaAppVersion",
  "$cordovaFile",
  "$cordovaFileTransfer",
  "$cordovaFileOpener2",
  "$cordovaDialogs",
  "$cordovaToast",
  "$cordovaInAppBrowser",
  "$interval",
  "cordovaService",
  function (
    $cordovaGeolocation,
    $cordovaSocialSharing,
    $cordovaAppVersion,
    $cordovaFile,
    $cordovaFileTransfer,
    $cordovaFileOpener2,
    $cordovaDialogs,
    $cordovaToast,
    $cordovaInAppBrowser,
    $interval,
    cordovaService
  ) {
    // -------------------------------------------------------------------------
    // Toast
    var showToast = function (message, duration, position) {
      if (!duration) {
        duration = "long";
      }
      if (!position) {
        position = "bottom";
      }
      $cordovaToast.show(message, duration, position);
    };

    // -------------------------------------------------------------------------
    // $cordovaDialogs
    var showDailogConfirm = function (
      message,
      title,
      buttonArray,
      buttonCallbackArray
    ) {
      $cordovaDialogs
        .confirm(message, title, buttonArray, buttonCallbackArray)
        .then(function (buttonIndex) {
          // no button = 0, 'OK' = 1, 'Cancel' = 2
          switch (buttonIndex) {
            case 0:
              break;
            case 1:
              buttonCallbackArray[0]();
              break;
            case 2:
              buttonCallbackArray[1]();
              break;
          }
        });
    };

    // -------------------------------------------------------------------------
    // Geolocation
    var getCurrentLocationByGeolocation = function (callback) {
      var posOptions = { timeout: 10000, enableHighAccuracy: true };
      $cordovaGeolocation.getCurrentPosition(posOptions).then(
        function (position) {
          console.log("getCurrentPosition. position = " + position);
          if (callback && callback.onSuccess) {
            callback.onSuccess(position);
          }
        },
        function (err) {
          if (callback && callback.onError) {
            callback.onError(err);
          }
        }
      );
    };
    var getCurrentLocationByBaiduLBS = function (callback) {
      var newCallback = {
        onSuccess: function (location) {
          location.coords = {
            latitude: undefined,
            longitude: undefined,
          };
          if (location) {
            location.coords.latitude = location.latitude;
            location.coords.longitude = location.longitude;
          }
          if (callback && callback.onSuccess) {
            callback.onSuccess(location);
          }
        },
        onError: callback ? callback.onError : undefined,
      };
      cordovaService.getCurrentLocationByBaiduLBS(newCallback);
    };
    var getCurrentLocationByHtml5 = function (callback) {
      if (navigator.geolocation) {
        var options = {
          enableHighAccuracy: true,
          timeout: 10000,
        };
        navigator.geolocation.getCurrentPosition(
          callback ? callback.onSuccess : null,
          callback ? callback.onError : null,
          options
        );
      } else {
        if (callback && callback.onError) {
          callback.onError({
            code: -1000,
            message: "很抱歉，您的设备不支持地理定位功能。",
          });
        }
      }
    };
    var getCurrentLocation = function (callback) {
      if (ionic.Platform.isAndroid()) {
        // getCurrentLocationByBaiduLBS(callback);
        getCurrentLocationByHtml5(callback);
      } else if (ionic.Platform.isIPad() || ionic.Platform.isIOS()) {
        getCurrentLocationByGeolocation(callback);
      } else {
        getCurrentLocationByGeolocation(callback);
      }
    };

    // -------------------------------------------------------------------------
    // SocialSharing
    var socialShare = function (
      message,
      subject,
      fileOrFileArray,
      url,
      callback
    ) {
      $cordovaSocialSharing
        .share(message, subject, fileOrFileArray, url) // Share via native share sheet
        .then(
          function (result) {
            // Success!
            if (callback && callback.onSuccess) {
              callback.onSuccess(result);
            }
          },
          function (err) {
            // An error occured. Show a message to the user
            if (callback && callback.onError) {
              callback.onError(err);
            }
          }
        );
    };

    // -------------------------------------------------------------------------
    // file operations: $cordovaFile $cordovaFileTransfer
    var checkFile = function (path, file, callback) {
      $cordovaFile.checkFile(path, file).then(
        function (result) {
          // success
          if (callback && callback.onSuccess) {
            callback.onSuccess(result);
          }
        },
        function (err) {
          // error
          if (callback && callback.onError) {
            callback.onError(err);
          }
        }
      );
    };
    var readAsText = function (path, file, callback) {
      $cordovaFile.readAsText(path, file).then(
        function (result) {
          // success
          if (callback && callback.onSuccess) {
            callback.onSuccess(result);
          }
        },
        function (err) {
          // error
          if (callback && callback.onError) {
            callback.onError(err);
          }
        }
      );
    };
    var readAsJson = function (path, file, callback) {
      $cordovaFile.readAsText(path, file).then(
        function (result) {
          // success
          result = angular.fromJson(result);
          if (callback && callback.onSuccess) {
            callback.onSuccess(result);
          }
        },
        function (err) {
          // error
          if (callback && callback.onError) {
            callback.onError(err);
          }
        }
      );
    };
    var writeFile = function (path, file, data, replace, callback) {
      $cordovaFile.writeFile(path, file, data, true).then(
        function (result) {
          // success
          if (callback && callback.onSuccess) {
            callback.onSuccess(result);
          }
        },
        function (error) {
          // error
          if (callback && callback.onError) {
            callback.onError(error);
          }
        }
      );
    };
    var writeJsonToFile = function (path, file, jsonObj, replace, callback) {
      var data = angular.toJson(jsonObj);
      writeFile(path, file, data, replace, callback);
    };
    var removeFile = function (path, file, callback) {
      $cordovaFile.removeFile(path, file).then(
        function (result) {
          // success
          if (callback && callback.onSuccess) {
            callback.onSuccess(result);
          }
        },
        function (error) {
          // error
          if (callback && callback.onError) {
            callback.onError(error);
          }
        }
      );
    };
    var download = function (url, targetPath, callback) {
      var trustHosts = true;
      var options = {};

      $cordovaFileTransfer.download(url, targetPath, options, trustHosts).then(
        function (result) {
          // Success!
          if (callback && callback.onSuccess) {
            callback.onSuccess(result);
          }
        },
        function (err) {
          // Error
          if (callback && callback.onError) {
            callback.onError(err);
          }
        },
        function (progress) {
          if (callback && callback.onProgress) {
            callback.onProgress(progress);
          }
        }
      );
    };
    var openFile = function (fileArg, contentType, callback) {
      $cordovaFileOpener2.open(fileArg, contentType).then(
        function () {
          // file opened successfully
          if (callback && callback.onSuccess) {
            callback.onSuccess();
          }
        },
        function (err) {
          // An error occurred. Show a message to the user
          if (callback && callback.onError) {
            callback.onError(err);
          }
        }
      );
    };
    var installAndroidApk = function (fileArg, callbackContext) {
      var contentType = "application/vnd.android.package-archive";
      openFile(fileArg, contentType, callbackContext);
    };

    // -------------------------------------------------------------------------
    // $cordovaAppVersion
    var getAppVersionNumber = function (callback) {
      $cordovaAppVersion.getVersionNumber().then(
        function (versionNumber) {
          // Success!
          if (callback && callback.onSuccess) {
            callback.onSuccess(versionNumber);
          }
        },
        function (err) {
          // An error occured. Show a message to the user
          if (callback && callback.onError) {
            callback.onError(err);
          }
        }
      );
    };
    var getAppVersionCode = function (callback) {
      $cordovaAppVersion.getVersionCode().then(
        function (versionCode) {
          // Success!
          if (callback && callback.onSuccess) {
            callback.onSuccess(versionCode);
          }
        },
        function (err) {
          // An error occured. Show a message to the user
          if (callback && callback.onError) {
            callback.onError(err);
          }
        }
      );
    };
    var getAppName = function (callback) {
      $cordovaAppVersion.getAppName().then(
        function (appName) {
          // Success!
          if (callback && callback.onSuccess) {
            callback.onSuccess(appName);
          }
        },
        function (err) {
          // An error occured. Show a message to the user
          if (callback && callback.onError) {
            callback.onError(err);
          }
        }
      );
    };
    var getAppPackageName = function (callback) {
      $cordovaAppVersion.getPackageName().then(
        function (packageName) {
          // Success!
          if (callback && callback.onSuccess) {
            callback.onSuccess(packageName);
          }
        },
        function (err) {
          // An error occured. Show a message to the user
          if (callback && callback.onError) {
            callback.onError(err);
          }
        }
      );
    };

    // $cordovaInAppBrowser
    var openUrlInAppBrowser = function (URL, callback) {
      var options = {
        location: "yes",
        clearcache: "yes",
        toolbar: "no",
      };
      $cordovaInAppBrowser
        .open(URL, "_blank", options)
        .then(function (event) {
          // success
        })
        .catch(function (event) {
          // error
        });
    };
    var closeInAppBrowser = function () {
      $cordovaInAppBrowser.close();
    };

    return {
      //
      showToast: showToast,
      //
      showDailogConfirm: showDailogConfirm,
      //
      getCurrentLocation: getCurrentLocation,
      //
      socialShare: socialShare,
      //
      checkFile: checkFile,
      readAsText: readAsText,
      readAsJson: readAsJson,
      writeFile: writeFile,
      writeJsonToFile: writeJsonToFile,
      removeFile: removeFile,
      download: download,
      openFile: openFile,
      installAndroidApk: installAndroidApk,
      //
      getAppVersionNumber: getAppVersionNumber,
      getAppVersionCode: getAppVersionCode,
      getAppName: getAppName,
      getAppPackageName: getAppPackageName,
      //
      openUrlInAppBrowser: openUrlInAppBrowser,
      closeInAppBrowser: closeInAppBrowser,
    };

    // // 基本变量和获取地理位置的方法
    // var getGeolocationDelay = 1000 * 60 * 30; // 每30分钟获取一次
    // var getGeolocationDelayPromise = null; // 记录每次开始重复获取地理位置的promise
    // var getGeolocationCallback = null; // 返回地理位置的对外接口
    // var getGeolocationDelayFunction = function () {
    //   var posOptions = { timeout: 10000, enableHighAccuracy: false };
    //   $cordovaGeolocation
    //     .getCurrentPosition(posOptions)
    //     .then(function (position) {
    //       console.log('getCurrentPosition. position = ' + position);
    //       if (getGeolocationCallback && getGeolocationCallback.onSuccess) {
    //         getGeolocationCallback.onSuccess(position);
    //       }
    //     }, function (err) {
    //       if (getGeolocationCallback && getGeolocationCallback.onError) {
    //         getGeolocationCallback.onError(err);
    //       }
    //     });
    // };
    // // 开始重复获取地理位置
    // var intervalGetGeolocation = function (callbak) {
    //   // 重新开始之前先尝试停止
    //   if (getGeolocationDelayPromise) {
    //     cancelIntevalGetGeolocation();
    //   }
    //   //
    //   getGeolocationCallback = callbak;
    //   getGeolocationDelayPromise = $interval(getGeolocationDelayFunction, getGeolocationDelay);
    //   console.log('begin getGeolocation repeat ...');
    // };
    // // 停止重复获取地理位置
    // var cancelIntevalGetGeolocation = function () {
    //   //
    //   if (!getGeolocationDelayPromise) {
    //     console.warn('there is no getGeolocation repeat task');
    //     return;
    //   }
    //   //
    //   $interval.cancel(getGeolocationDelayPromise);
    //   getGeolocationDelayPromise = null;
    //   // $interval.flush(); // 不要执行该方法，否则将会阻塞程序的运行。
    //   console.log('stop getGeolocation repeat ...');
    // };

    // return {
    //   intervalGetGeolocation: intervalGetGeolocation,
    //   cancelIntevalGetGeolocation: cancelIntevalGetGeolocation,
    // };
  },
]);
