angular
  .module("starter.controllers")

  .controller(
    "DiscoveryCtrl",
    function (
      $rootScope,
      $scope,
      $state,
      $ionicHistory,
      $ionicSlideBoxDelegate,
      $ionicTabsDelegate,
      $ionicLoading,
      apiService,
      uiService,
      ngCordovaService,
      $ionicHistory
    ) {
      // // 由于该状态是从'tab.mine'跳过来的，再在app.js中通过$stateChangeStart处理，会出现问题。故手动处理如下。
      if (!$scope.user) {
        $state.go("tab.mine", { fromState: $ionicHistory.currentStateName() });
        return;
      }

      $scope.tabNames = ["书房", "图书"];
      $scope.slectIndex = 0;
      $scope.activeSlide = function (index) {
        //点击时候触发
        $scope.slectIndex = index;
        $ionicSlideBoxDelegate.slide(index);
      };
      $scope.slideChanged = function (index) {
        //滑动时候触发
        $scope.slectIndex = index;
      };
      $scope.pages = [
        "modules/discovery/dc_homes.html",
        "modules/discovery/dc_books.html",
      ];

      // -------------------------------------------------------------------------------------------------------------------
      // $scope.persons = null; // 书房列表
      // $scope.books = null; // 图书列表
      // 注释掉上目的内容，即不重写$scope，而是使用全局的$rootScope，实在是无奈之举。
      // 否则，从第二次进入“发现”开始，数据是得到了，但是写不到当前页面的$scope的上述属性中，到这页面显示为空。原因未知。

      // -------------------------------------------------------------------------------------------------------------------
      // 刷新页面数据
      $scope.refreshAll = function () {
        uiService.ionicLoadingShow({ templateTitle: "正在查询..." }); // 隐藏loading的时机：1.获取地理位置失败；2.上传地理位置失败；3.获取附近的书房和图书完成（不只是成功或者失败）
        $scope.getCurrentLocation();
      };
      // 获取地理位置
      $scope.getCurrentLocation = function () {
        // uiService.ionicLoadingShow({ templateTitle: '正在获取您的地理位置信息...' });
        ngCordovaService.getCurrentLocation(getCurrentLocationCallback);
      };
      var getCurrentLocationCallback = {
        onSuccess: function (location) {
          var latitude = location.coords.latitude;
          var longitude = location.coords.longitude;
          console.log("location. " + "(" + latitude + ", " + longitude + ")");
          $scope.uploadPersonLocation(location);
        },
        onError: function (err) {
          console.warn(
            "无法获取当前位置，请检查是否已经打开GPS开关。error is \n" +
              angular.toJson(err, true)
          );
          $ionicLoading.hide();
          var error_msg = null;
          switch (err.code) {
            case err.PERMISSION_DENIED:
              error_msg = "您拒绝了地理定位请求";
              break;
            case err.POSITION_UNAVAILABLE:
              error_msg = "位置信息不可用.";
              break;
            case err.TIMEOUT:
              error_msg = "获取用户位置超时.";
              break;
            case err.UNKNOWN_ERROR:
              error_msg = "获取地理位置的时候出现了未知异常";
              break;
            default:
              error_msg = err.message;
              break;
          }
          ngCordovaService.showToast(error_msg);
          // // TODO 如果手机不支持GPS点位获取，可以打开下面的注释。
          // var location = {
          //     coords: {
          //         longitude: '120.4102046944',
          //         latitude: '36.1795879094',
          //     }
          // };
          // $scope.uploadPersonLocation(location);
        },
        onFinally: function () {},
      };
      // 上传地理位置
      $scope.uploadPersonLocation = function (location) {
        // uiService.ionicLoadingShow({ templateTitle: '正在上次您的地理位置信息...' });
        var peopleLocation = {
          userId: $scope.user.account,
          longitude: location.coords.longitude,
          latitude: location.coords.latitude,
        };
        apiService.uploadPersonLocation(
          peopleLocation,
          uploadPersonLocationCallback
        );
      };
      var uploadPersonLocationCallback = {
        onSuccess: function (data, status, config, headers) {
          $scope.getNearbyPeopleLibrooms();
          $scope.getRecentBooks();
        },
        onError: function (data, status, config, headers) {
          ngCordovaService.showToast("提交当前位置失败");
          // ngCordovaService.showToast('上传地理位置信息失败');
          $ionicLoading.hide();
        },
        onFinally: function () {},
      };
      // GET 查询周围用户书房信息
      $scope.getNearbyPeopleLibrooms = function () {
        $scope.isGettingNearbyPeopleLibrooms = true;
        var userId = $scope.user.account;
        var radius = 1000;
        var count = 10;
        apiService.getNearbyPeopleLibrooms(
          userId,
          radius,
          count,
          getNearbyPeopleLibroomsCallback
        );
      };
      var getNearbyPeopleLibroomsCallback = {
        onSuccess: function (data, status, config, headers) {
          $rootScope.persons = data;
          for (var i = 0; i < $scope.persons.length; i++) {
            $scope.persons[i].userId = $scope.persons[i].member;
            // TODO 处理附近书房的头像问题
            if (!$scope.persons[i].photo_url) {
              $scope.persons[i].photo_url =
                baseServerUrl + "/resources/head_images/mpic/default.jpg";
            }
          }
        },
        onError: function (data, status, config, headers) {
          ngCordovaService.showToast("获取附近的书房失败");
        },
        onFinally: function () {
          $scope.isGettingNearbyPeopleLibrooms = false;
          if ($scope.isGettingRecentBooks == true) {
            $ionicLoading.hide();
          }
        },
      };
      // GET 查询周围用户书房信息
      $scope.getRecentBooks = function () {
        $scope.isGettingRecentBooks = true;
        var userId = $scope.user.account;
        var count = 10;
        apiService.getRecentBooks(userId, count, getRecentBooksCallback);
      };
      var getRecentBooksCallback = {
        onSuccess: function (data, status, config, headers) {
          $rootScope.books = data;
        },
        onError: function (data, status, config, headers) {
          ngCordovaService.showToast("获取附近的书房失败");
        },
        onFinally: function () {
          $scope.isGettingRecentBooks = false;
          if ($scope.isGettingNearbyPeopleLibrooms == true) {
            $ionicLoading.hide();
          }
        },
      };

      // -------------------------------------------------------------------------------------------------------------------
      $scope.goBookHome = function (person) {
        $ionicHistory.clearCache().then(function () {
          $state.go("tab.dcbhdetail", {
            person: person,
          });
        });
      };
      // -------------------------------------------------------------------------------------------------------------------
      $scope.goBookDetailDrift = function (book) {
        $state.go("tab.dcbookdetaildrift", {
          book: book,
        });
      };

      // -------------------------------------------------------------------------------------------------------------------
      $ionicTabsDelegate.showBar(true);
      $scope.refreshAll();
    }
  );
