angular
  .module("starter.controllers")
  .controller(
    "HomeCtrl",
    function (
      $cordovaBarcodeScanner,
      $rootScope,
      $scope,
      $state,
      apiService,
      bookService,
      ngCordovaService,
      $ionicTabsDelegate,
      $ionicHistory,
      $ionicPopup,
      broadcastConstants,
      broadcastService,
      uiService,
      $ionicLoading,
      $ionicPlatform
    ) {
      $ionicHistory.clearHistory();
      /*
       *
       * （图书）数据缓存流程
       * 1. 判断当前scope（实际是rootScope）中是否存在对应的数据；
       * 2. 如果存在数据，说明之前曾经成功获取过数据（，且之前应当已经将该数据写入到缓存文件），直接使用数据即可，原因见步骤6；
       * 3. 如果数据不存在，则需要重新获取；
       * 4. 先检查缓存文件，如果可以从缓存文件中获取数据，则先从使用此缓存数据；然后，再重新从远程获取数据；
       * 5. 如果无法从缓存文件中获取数据，则直接从远程获取数据；
       * 6. 从远程获取数据之后，要把该数据写入到rootScope中，方便下次切换到该状态直接使用；并且，还要将该数据写入到缓存文件中，方便下次需要的时候从缓存文件获取数据；
       * 注：步骤4和5都会从远程获取数据，步骤4的重点不在从远程获取数据，而在于从缓存文件获取数据使用，目的是：1）加快显示速度；2）在未联网的情况下，如故上一次曾经成功获取过数据，可以离线使用已有的数据。
       *
       */

      //
      $scope.onGetNewBooksLoading = false;
      $scope.onGetGoodBooksLoading = false;
      $scope.newRecommendedBooksCacheFileName = "newRecommendedBooks.json";
      $scope.mostRecommendedBooksCacheFileName = "mostRecommendedBooks.json";

      $scope.slideImageUrls = undefined;
      /**
       * 对于ion-slide-box控件，经过多次验证，发现，其数据只有第一次有效，重新改变数据之后，无非重新生效。
       * 下面的代码逻辑是：
       * 1. 如果能够从localStorage中找到，则使用localStorage存储的值
       * 2. 否则
       * 2.1 先使用默认值
       * 2.2 再从网络获取
       * 注意：
       * 1. 在视图中，不要使用$rootScope中的变量定义，而要使用$scope中的变量定义，否则在首次打开APP首页的时候，无法正常显示滚动图片。
       */
      // 检查是否存在
      if (_.isEmpty($scope.slideImageUrls)) {
        // 1. 从localStorage中获取
        try {
          var value = localStorage.getItem("homePageSlideImageUrls");
          if (value) {
            var imageUrls = value.split(","); // 以“,”分割
            if (!_.isEmpty(imageUrls)) {
              var imageUrls2 = [];
              for (var index = 0; index < imageUrls.length; index++) {
                var url = imageUrls[index];
                if (!_.isEmpty(url)) {
                  imageUrls2.push(url);
                }
              }
              if (!_.isEmpty(imageUrls2)) {
                // $rootScope.homePageSlideImageUrls = imageUrls2; // 在controller中，为了防止异步操作时间差问题（app.js中步骤3也会处理），这里不再设置。
                console.log(
                  "从localStorage获取首页滚动图片：\n" +
                    angular.toJson($rootScope.homePageSlideImageUrls)
                );
                $scope.slideImageUrls = imageUrls2;
              } else {
                console.warn("从localStorage获取首页滚动图片：为空");
              }
            }
          }
        } catch (error) {
          console.error(
            "从localStorage获取首页滚动图片：出现异常，" + angular.toJson(error)
          );
        }
      }
      // 再检查一遍
      if (_.isEmpty($scope.slideImageUrls)) {
        // 2. 使用默认的图片
        $scope.slideImageUrls = [
          "img/home/roll_1.jpg",
          "img/home/roll_2.jpg",
          "img/home/roll_3.jpg",
          "img/home/roll_4.jpg",
        ];
        console.log("首页滚动图片，使用默认图片。");
      }

      // ----------------------------------------------------------------------------------------------------------------
      // 从缓存文件中获取新书推荐
      $scope.getNewRecommendedBooksFromCacheFileAndRemote = function () {
        console.log("从缓存文件中获取新书推荐 ...");
        if (ionic.Platform.isIPad() || ionic.Platform.isIOS()) {
          // IOS
          console.log("从缓存文件中获取新书推荐 ... IOS平台 ...");
          ngCordovaService.readAsJson(
            cordova.file.applicationStorageDirectory,
            $scope.newRecommendedBooksCacheFileName,
            getNewRecommendedBooksFromCacheFileAndRemoteCallback
          );
        } else if (ionic.Platform.isAndroid()) {
          // Android
          console.log("从缓存文件中获取新书推荐 ... Android平台 ...");
          ngCordovaService.readAsJson(
            cordova.file.externalApplicationStorageDirectory,
            $scope.newRecommendedBooksCacheFileName,
            getNewRecommendedBooksFromCacheFileAndRemoteCallback
          );
        } else {
          // Others are not supported
          console.log("当前平台暂不支持从缓存文件中获取新书推荐");
          $scope.retrieveNewRecommendedBooks();
        }
      };
      var getNewRecommendedBooksFromCacheFileAndRemoteCallback = {
        onSuccess: function (data) {
          console.log("从缓存文件中获取新书推荐 ... Android平台 ... 成功");
          $scope.newBooksRows = data; // 1. 如果可以从缓存文件中读取，可以优先显示，提高效率。
          $scope.retrieveNewRecommendedBooks(); // 2. 然后重新从网络获取，并更新rootScope。
        },
        onError: function (err) {
          console.warn(
            "从缓存文件中获取新书推荐 ... Android平台 ... 失败。 error is " +
              "{code:" +
              err.code +
              ", message:" +
              err.message +
              "}"
          );
          $scope.retrieveNewRecommendedBooks(); // 3. 如果无法从缓存文件中读取，显然需要从网络获取，并更新rootScope。
        },
      };
      // 从缓存文件中获取好书推荐
      $scope.getMostRecommendedBooksFromCacheFileAndRemote = function () {
        console.log("从缓存文件中获取好书推荐 ...");
        if (ionic.Platform.isIPad() || ionic.Platform.isIOS()) {
          // IOS
          console.log("从缓存文件中获取好书推荐 ... IOS平台 ...");
          ngCordovaService.readAsJson(
            cordova.file.applicationStorageDirectory,
            $scope.mostRecommendedBooksCacheFileName,
            getMostRecommendedBooksFromCacheFileAndRemoteCallback
          );
        } else if (ionic.Platform.isAndroid()) {
          // Android
          console.log("从缓存文件中获取好书推荐 ... Android平台 ...");
          ngCordovaService.readAsJson(
            cordova.file.externalApplicationStorageDirectory,
            $scope.mostRecommendedBooksCacheFileName,
            getMostRecommendedBooksFromCacheFileAndRemoteCallback
          );
        } else {
          // Others are not supported
          console.log("当前平台暂不支持从缓存文件中获取好书推荐");
          $scope.retrieveMostRecommendedBooks();
        }
      };
      var getMostRecommendedBooksFromCacheFileAndRemoteCallback = {
        onSuccess: function (data) {
          console.log("从缓存文件中获取好书推荐 ... Android平台 ... 成功");
          $scope.goodBooksRows = data; // 1. 如果可以从缓存文件中读取，可以优先显示，提高效率。
          $scope.retrieveMostRecommendedBooks(); // 2. 然后重新从网络获取，并更新rootScope。
        },
        onError: function (err) {
          console.warn(
            "从缓存文件中获取好书推荐 ... Android平台 ... 失败。 error is " +
              "{code:" +
              err.code +
              ", message:" +
              err.message +
              "}"
          );
          $scope.retrieveMostRecommendedBooks(); // 3. 如果无法从缓存文件中读取，显然需要从网络获取，并更新rootScope。
        },
      };

      // ---------------------------------------------------------------------------------------------------------------
      // 获取新书推荐
      $scope.retrieveNewRecommendedBooks = function () {
        $scope.account = "";
        if ($rootScope.user) {
          $scope.account = $rootScope.user.account;
        }
        $scope.onGetNewBooksLoading = true;
        apiService.retrieveNewRecommendedBooks($scope.account, 0, 10, {
          onSuccess: onGetNewBooksSuccess,
          onError: onError,
          onFinally: function () {
            $scope.onGetNewBooksLoading = false;
          },
        });
      };
      // 获取好书推荐
      $scope.retrieveMostRecommendedBooks = function () {
        $scope.account = "";
        if ($rootScope.user) {
          $scope.account = $rootScope.user.account;
        }
        $scope.onGetGoodBooksLoading = true;
        apiService.retrieveMostRecommendedBooks($scope.account, 0, 10, {
          onSuccess: onGetGoodBooksSuccess,
          onError: onError,
          onFinally: function () {
            $scope.onGetGoodBooksLoading = false;
          },
        });
      };

      // ----------------------------------------------------------------------------------------------------------------
      // 获取新书成功的回调
      function onGetNewBooksSuccess(data) {
        //
        var books = data.data;
        if (books) {
          $scope.firstPageNewBooks = books.slice(0);
        }
        bookService.fixBooks(books);
        //
        var size = books.length;
        var colCount = 3;
        var rowCount = Math.ceil(size / colCount);
        rowCount = 2; // server端有时返回的图书个数可能不对，app要求只显示2行，所以这里强制设置为2.
        var rows = oneDimToTwoDimArray(books, rowCount, colCount);
        $scope.newBooksRows = rows;
        // save as const
        $rootScope.newBooksRows = $scope.newBooksRows;
        // write cache file
        console.log("将新书推荐写入到缓存文件中 ...");
        if (ionic.Platform.isIPad() || ionic.Platform.isIOS()) {
          // IOS
          console.log("将新书推荐写入到缓存文件中 ... IOS平台 ...");
          ngCordovaService.writeJsonToFile(
            cordova.file.applicationStorageDirectory,
            $scope.newRecommendedBooksCacheFileName,
            $scope.newBooksRows,
            true,
            {
              onSuccess: function (data) {
                console.log("已经将新书推荐的数据写入到缓存文件中.");
              },
              onError: function (err) {
                console.log(
                  "在将新书推荐的数据写入到缓存文件的过程中出现错误. error is " +
                    "{code:" +
                    err.code +
                    ", message:" +
                    err.message +
                    "}"
                );
              },
            }
          );
        } else if (ionic.Platform.isAndroid()) {
          // Android
          console.log("将新书推荐写入到缓存文件中 ... Android平台 ...");
          ngCordovaService.writeJsonToFile(
            cordova.file.externalApplicationStorageDirectory,
            $scope.newRecommendedBooksCacheFileName,
            $scope.newBooksRows,
            true,
            {
              onSuccess: function (data) {
                console.log("已经将新书推荐的数据写入到缓存文件中.");
              },
              onError: function (err) {
                console.log(
                  "在将新书推荐的数据写入到缓存文件的过程中出现错误. error is " +
                    "{code:" +
                    err.code +
                    ", message:" +
                    err.message +
                    "}"
                );
              },
            }
          );
        } else {
          // Others are not supported
          console.log("当前平台暂不支持 将好书推荐写入到缓存文件中");
        }
      }
      // 获取好书成功的回调
      function onGetGoodBooksSuccess(data) {
        //
        var books = data.data;
        if (books) {
          $scope.firstPageGoodBooks = books.slice(0);
        }
        bookService.fixBooks(books);
        //
        var size = books.length;
        var colCount = 3;
        var rowCount = Math.ceil(size / colCount);
        rowCount = 2; // server端有时返回的图书个数可能不对，app要求只显示2行，所以这里强制设置为2.
        var rows = oneDimToTwoDimArray(books, rowCount, colCount);
        $scope.goodBooksRows = rows;
        // save as const
        $rootScope.goodBooksRows = $scope.goodBooksRows;
        // write cache file
        console.log("将好书推荐写入到缓存文件中 ...");
        if (ionic.Platform.isIPad() || ionic.Platform.isIOS()) {
          // IOS
          console.log("将好书推荐写入到缓存文件中 ... IOS平台 ...");
          ngCordovaService.writeJsonToFile(
            cordova.file.applicationStorageDirectory,
            $scope.mostRecommendedBooksCacheFileName,
            $scope.goodBooksRows,
            true,
            {
              onSuccess: function (data) {
                console.log("已经将好书推荐的数据写入到缓存文件中.");
              },
              onError: function (err) {
                console.log(
                  "在将好书推荐的数据写入到缓存文件的过程中出现错误. error is " +
                    "{code:" +
                    err.code +
                    ", message:" +
                    err.message +
                    "}"
                );
              },
            }
          );
        } else if (ionic.Platform.isAndroid()) {
          // Android
          console.log("将好书推荐写入到缓存文件中 ... Android平台 ...");
          ngCordovaService.writeJsonToFile(
            cordova.file.externalApplicationStorageDirectory,
            $scope.mostRecommendedBooksCacheFileName,
            $scope.goodBooksRows,
            true,
            {
              onSuccess: function (data) {
                console.log("已经将好书推荐的数据写入到缓存文件中.");
              },
              onError: function (err) {
                console.log(
                  "在将好书推荐的数据写入到缓存文件的过程中出现错误. error is " +
                    "{code:" +
                    err.code +
                    ", message:" +
                    err.message +
                    "}"
                );
              },
            }
          );
        } else {
          // Others are not supported
          console.log("当前平台暂不支持 将好书推荐写入到缓存文件中");
        }
      }
      function onError() {
        // TODO
      }

      // ----------------------------------------------------------------------------------------------------------------
      // 对数组进行行列设置：1维->2维
      function oneDimToTwoDimArray(arr, rowCount, colCount) {
        var size = arr.length;
        var rows = [];
        for (var row = 0; row < rowCount; row++) {
          rows[row] = [];
          for (var col = 0; col < colCount; col++) {
            var index = row * colCount + col;
            if (index < size) {
              rows[row][col] = arr[index];
            } else {
              break;
            }
          }
        }
        return rows;
      }

      // ----------------------------------------------------------------------------------------------------------------

      /*******************展示服务条款和隐私政策的弹窗***********************/
      $scope.showConfirm = function () {
        $rootScope.confirmPopup = $ionicPopup.confirm({
          title: "服务协议和隐私政策",
          template:
            "<div>请你务必审慎阅读、充分理解“服务协议”和“隐私政策”各条款，包括但不限于：为了向你提供即时通讯、内容分享等服务，我们需要收集你的设备信息、操作日志等个人信息。</div>" +
            "<div>你可以在“设置”中查看、变更、删除个人信息并管理你的授权。你可阅读" +
            "<a style='color:#387ef5;' href='#/tab/homeserviceAgreement'>《服务协议》</a>" +
            "和" +
            "<a style='color:#387ef5;' href='#/tab/homeprivacyPolicy'>《隐私政策》</a>" +
            "了解详细信息。如你同意，请点击“同意”开始接受我们的服务。</div>",
          buttons: [
            {
              // Array[Object] (optional). Buttons to place in the popup footer.
              text: "暂不使用",
              type: "button-default",
              onTap: function (e) {
                // e.preventDefault() will stop the popup from closing when tapped.
                ionic.Platform.exitApp();
              },
            },
            {
              text: "同意",
              type: "button-positive",
              onTap: function (e) {
                // Returning a value will cause the promise to resolve with the given value.
                // return scope.data.response;
                localStorage.setItem("savaAgreeClause", "agreeClause");
              },
            },
          ],
        });
        // confirmPopup.then(function(res) {
        //   backAction();
        // });
      };
      //禁止点击系统返回键使弹窗消失
      $scope.initShowClause = function () {
        var localAgreeClause = localStorage.getItem("savaAgreeClause");
        if (!localAgreeClause) {
          //只有在首次登录的时候才会出现服务条款的弹窗
          $scope.showConfirm();
        }
      };
      $scope.initShowClause();
      $scope.goDetail = function (book) {
        $ionicHistory.clearCache().then(function () {
          $state.go("tab.book-detail", { book: book });
        });
      };
      $scope.goMoreNewBooks = function () {
        $ionicHistory.clearCache().then(function () {
          $state.go("tab.newbook-more", {
            firstPageNewBooks: $scope.firstPageNewBooks,
          });
        });
      };
      $scope.goMoreGoodBooks = function () {
        $ionicHistory.clearCache().then(function () {
          $state.go("tab.goodbook-more", {
            firstPageGoodBooks: $scope.firstPageGoodBooks,
          });
        });
      };
      /****************wsc-change**********************/
      $scope.goReserveBook = function () {
        $ionicHistory.clearCache().then(function () {
          $state.go("tab.homeminereserve");
        });
      };
      $scope.goConditionalSearchBook = function () {
        $ionicHistory.clearCache().then(function () {
          $state.go("tab.conditinalSearchBook");
        });
      };
      $scope.goCategory = function () {
        $ionicHistory.clearCache().then(function () {
          $state.go("tab.category");
        });
      };
      $scope.goTopLoanedBooks = function () {
        $ionicHistory.clearCache().then(function () {
          $state.go("tab.top-loaned-books");
        });
      };
      $scope.showAlert = function () {
        var alertPopup = $ionicPopup.alert({
          //title: 'Don\'t eat that!',
          template: "该功能正在开发中，敬请期待",
          okText: "确定",
        });
      };
      /****************wsc-change***********************/
      $scope.goBorrowBook = function () {
        $ionicHistory.clearCache().then(function () {
          $state.go("tab.myborrowed");
        });
      };
      $scope.goActivityWorldMine = function () {
        $ionicHistory.clearCache().then(function () {
          $state.go("tab.activityworldmine");
        });
      };

      $scope.goReturnOrder = function () {
        $ionicHistory.clearCache().then(function () {
          $state.go("tab.homeorderreturn");
        });
      };
      /************wsc-change*****************/
      $scope.goDiscovery = function () {
        $ionicHistory.clearCache().then(function () {
          $state.go("tab.discovery");
        });
      };
      $scope.getBarcode = function () {
        $cordovaBarcodeScanner.scan().then(
          function (result) {
            if (!$rootScope.user) {
              ngCordovaService.showToast("请登录后尝试开门...");
            } else {
              if (result.text) {
                $scope.qrcodeArr = result.text.split("?");
                var scanParams = $scope.qrcodeArr[1];
                if (
                  $scope.qrcodeArr[0] ==
                  "http://wx.leituzhiyue.com/haiheng/qrcode_scaning"
                ) {
                  uiService.ionicLoadingShow({
                    templateTitle: "正在登录，请耐心等待...",
                  });
                  apiService.haihengScanQRcode(
                    $rootScope.user.account,
                    scanParams,
                    {
                      onSuccess: function (data, status, config, headers) {
                        if (data.status == "OK") {
                          $state.go("tab.homescanResult", {
                            scanResult: data.data,
                          });
                        } else {
                          ngCordovaService.showToast(data.data);
                        }
                      },
                      onError: function (data, status, config, headers) {
                        ngCordovaService.showToast("登录失败，请稍后重试...");
                        //
                      },
                      onFinally: function () {
                        $ionicLoading.hide();
                      },
                    }
                  );
                } else {
                  uiService.ionicLoadingShow({
                    templateTitle: "正在开门，请耐心等待...",
                  });
                  $scope.qrcode = scanParams.replace(
                    "WechatOpenDoor",
                    "AppOpenDoor"
                  );
                  apiService.scanBarcodeOpen(
                    $rootScope.user.account,
                    $rootScope.user.name,
                    $scope.qrcode,
                    {
                      onSuccess: function (data, status, config, headers) {
                        if (data.status == "OK") {
                          $state.go("tab.homescanResult", {
                            scanResult: data.data,
                          });
                        } else {
                          ngCordovaService.showToast(data.data);
                        }
                      },
                      onError: function (data, status, config, headers) {
                        ngCordovaService.showToast("开门失败，请稍后重试...");
                        //
                      },
                      onFinally: function () {
                        $ionicLoading.hide();
                      },
                    }
                  );
                }
              }
            }
          },
          function (error) {
            consolog.warn("Scanning failed: " + error);
            ngCordovaService.showToast("摄像头可能正在使用中，请稍后再试...");
          }
        );
      };

      $scope.electronicRead = function () {
        if (!$rootScope.user) {
          $state.go("tab.mine", {
            fromState: $ionicHistory.currentStateName(),
          });
          return;
        } else {
          apiService.electronicRead($rootScope.user.account, {
            onSuccess: function (data, status, config, headers) {
              if (data.status == "OK") {
                var url = data.data;
                cordova.ThemeableBrowser.open(url, "_blank", {
                  statusbar: {
                    color: "#003264ff",
                  },
                  toolbar: {
                    height: 44,
                    color: "#f0f0f0ff",
                  },
                  title: {
                    color: "#003264ff",
                    staticText: "电子资源",
                  },
                  backButton: {
                    image: "back",
                    imagePressed: "back_pressed",
                    align: "left",
                    event: "backPressed",
                  },

                  backButtonCanClose: true,
                })
                  .addEventListener("loadstart", function (e) {
                    navigator.notification.activityStart(
                      "请等待",
                      "正在打开页面"
                    );
                  })
                  .addEventListener("loadstop", function () {
                    navigator.notification.activityStop();
                  })
                  .addEventListener(
                    cordova.ThemeableBrowser.EVT_ERR,
                    function (e) {
                      navigator.notification.activityStop();
                      ngCordovaService.showToast(e.message);
                    }
                  );
              } else {
                ngCordovaService.showToast(data.data);
              }
            },
            onError: function (data, status, config, headers) {
              ngCordovaService.showToast("服务器连接失败，请稍后尝试");
            },
            onFinally: function () {},
          });
        }
      };
      $scope.getOnlineBookStore = function () {
        if (!$rootScope.user) {
          $state.go("tab.mine", {
            fromState: $ionicHistory.currentStateName(),
          });
          return;
        } else {
          apiService.onlineBookstore($rootScope.user.account, {
            onSuccess: function (data, status, config, headers) {
              if (data.status == "OK") {
                var url = data.data;
                cordova.ThemeableBrowser.open(url, "_blank", {
                  statusbar: {
                    color: "#003264ff",
                  },
                  toolbar: {
                    height: 44,
                    color: "#f0f0f0ff",
                  },
                  title: {
                    color: "#003264ff",
                    staticText: "在线书城",
                  },
                  backButton: {
                    image: "back",
                    imagePressed: "back_pressed",
                    align: "left",
                    event: "backPressed",
                  },

                  backButtonCanClose: true,
                })
                  .addEventListener("loadstart", function (e) {
                    navigator.notification.activityStart(
                      "请等待",
                      "正在打开页面"
                    );
                  })
                  .addEventListener("loadstop", function () {
                    navigator.notification.activityStop();
                  })
                  .addEventListener(
                    cordova.ThemeableBrowser.EVT_ERR,
                    function (e) {
                      navigator.notification.activityStop();
                      ngCordovaService.showToast(e.message);
                    }
                  );
              } else {
                ngCordovaService.showToast(data.data);
              }
            },
            onError: function (data, status, config, headers) {
              ngCordovaService.showToast("服务器连接失败，请稍后尝试");
            },
            onFinally: function () {},
          });
        }
      };
      $scope.goElectronicJournal = function () {
        if (!$rootScope.user) {
          $state.go("tab.mine", {
            fromState: $ionicHistory.currentStateName(),
          });
          return;
        } else {
          apiService.electronicJournal($rootScope.user.account, {
            onSuccess: function (data, status, config, headers) {
              if (data.status == "OK") {
                var url = data.data;
                cordova.ThemeableBrowser.open(url, "_blank", {
                  statusbar: {
                    color: "#003264ff",
                  },
                  toolbar: {
                    height: 44,
                    color: "#f0f0f0ff",
                  },
                  title: {
                    color: "#003264ff",
                    staticText: "电子期刊",
                  },
                  backButton: {
                    image: "back",
                    imagePressed: "back_pressed",
                    align: "left",
                    event: "backPressed",
                  },

                  backButtonCanClose: true,
                })
                  .addEventListener("loadstart", function (e) {
                    navigator.notification.activityStart(
                      "请等待",
                      "正在打开页面"
                    );
                  })
                  .addEventListener("loadstop", function () {
                    navigator.notification.activityStop();
                  })
                  .addEventListener(
                    cordova.ThemeableBrowser.EVT_ERR,
                    function (e) {
                      navigator.notification.activityStop();
                      ngCordovaService.showToast(e.message);
                    }
                  );
              } else {
                ngCordovaService.showToast(data.data);
              }
            },
            onError: function (data, status, config, headers) {
              ngCordovaService.showToast("服务器连接失败，请稍后尝试");
            },
            onFinally: function () {},
          });
        }
      };

      $scope.goElectronicAudio = function () {
        if (!$rootScope.user) {
          $state.go("tab.mine", {
            fromState: $ionicHistory.currentStateName(),
          });
          return;
        } else {
          apiService.longyuanAudio($rootScope.user.account, {
            onSuccess: function (data, status, config, headers) {
              if (data.status == "OK") {
                var url = data.data;
                cordova.ThemeableBrowser.open(url, "_blank", {
                  statusbar: {
                    color: "#003264ff",
                  },
                  toolbar: {
                    height: 44,
                    color: "#f0f0f0ff",
                  },
                  title: {
                    color: "#003264ff",
                    staticText: "电子音频",
                  },
                  backButton: {
                    image: "back",
                    imagePressed: "back_pressed",
                    align: "left",
                    event: "backPressed",
                  },

                  backButtonCanClose: true,
                })
                  .addEventListener("loadstart", function (e) {
                    navigator.notification.activityStart(
                      "请等待",
                      "正在打开页面"
                    );
                  })
                  .addEventListener("loadstop", function () {
                    navigator.notification.activityStop();
                  })
                  .addEventListener(
                    cordova.ThemeableBrowser.EVT_ERR,
                    function (e) {
                      navigator.notification.activityStop();
                      ngCordovaService.showToast(e.message);
                    }
                  );
              } else {
                ngCordovaService.showToast(data.data);
              }
            },
            onError: function (data, status, config, headers) {
              ngCordovaService.showToast("服务器连接失败，请稍后尝试");
            },
            onFinally: function () {},
          });
        }
      };

      // ----------------------------------------------------------------------------------------------------------------
      // 为防止部分变量加载延迟，所以做监听器处理。
      document.addEventListener("deviceready", onDeviceReady, false);
      function onDeviceReady() {
        console.log("cordova.file:" + cordova.file);
        //
        if (!$rootScope.user) {
          // 注册广播 当有用户登录通知的时候，重新从网络获取
          var onuserInfoChangeListener = function () {
            $scope.retrieveNewRecommendedBooks();
            $scope.retrieveMostRecommendedBooks();
          };
          broadcastService.register(
            broadcastConstants.names.userInfoChange,
            onuserInfoChangeListener
          );

          // 取消广播
          $scope.$on("$destroy", function () {
            broadcastService.unregister(
              broadcastConstants.names.userInfoChange,
              onuserInfoChangeListener
            );
          });
        }
        // 执行数据获取
        if (!$scope.newBooksRows || $rootScope.user) {
          $scope.getNewRecommendedBooksFromCacheFileAndRemote();
        }
        if (!$scope.goodBooksRows || $rootScope.user) {
          $scope.getMostRecommendedBooksFromCacheFileAndRemote();
        }
      }

      // ----------------------------------------------------------------------------------------------------------------
      $ionicTabsDelegate.showBar(true);
    }
  );
