angular
  .module("starter.controllers")

  // 图书搜索
  .controller(
    "SearchBookCtrl",
    function (
      $rootScope,
      $scope,
      $stateParams,
      $state,
      $ionicLoading,
      apiService,
      bookService,
      uiService,
      $ionicTabsDelegate,
      appLogService
    ) {
      // ----------------------------------------------------------------------------------------------------------------

      $scope.account;
      if (!$scope.user) {
        $scope.account = "";
      } else {
        $scope.account = $rootScope.userInfo.account;
      }
      // 消息
      $scope.refreshFailed = false;
      $scope.refreshFailedMsg = "刷新失败，请下拉重新";
      $scope.loadMoreFailed = false;
      $scope.loadMoreFailedMsg = "加载更多失败";
      $scope.noMoreBooksMsg = "已经加载所有图书";
      // ----------------------------------------------------------------------------------------------------------------
      // 路由参数
      $scope.category = $stateParams.category; // 分类
      $scope.title = $scope.category.name; // 分类名称
      // 查询参数与结果
      $scope.books = []; // 图书集合
      $scope.categoryCode = $scope.category.code; // 分类码
      $scope.elementsCount = 0; // 当前查询结果集的总个数
      $scope.totalPages = -1; // 总页码 == Math.ceil(elementsCount/size) 这里设置总页码为-1
      $scope.page = -1; // 查询的当前页面  有效值范围：[0, totalPages - 1]
      $scope.size = 10; // 查询的每页图书本书

      // ----------------------------------------------------------------------------------------------------------------
      // 清除与重置数据
      $scope.clearData = function () {
        $scope.refreshFailed = false;
        $scope.loadMoreFailed = false;
        $scope.books = [];
        $scope.elementsCount = 0;
        $scope.totalPages = -1;
        $scope.page = -1;
      };
      // 第一次加载搜索结果  结果中包含结果集引用值以及总的搜索结果
      $scope.refresh = function () {
        // 清除与重置数据
        $scope.clearData();
        //
        $scope.searchBooksOfNextPageBycategory();
      };
      // 加载更多
      $scope.loadMore = function (ignoreLoadMoreFailed) {
        if ($scope.refreshFailed) {
          console.warn(
            "加载首页图书失败，加载下一页只能在之前加载成功的情况下进行。"
          );
          return;
        }
        if ($scope.loadMoreFailed) {
          if (ignoreLoadMoreFailed) {
          } else {
            console.warn(
              "上次加载更多图书失败，加载下一页只能在之前加载成功的情况下进行。"
            );
            return;
          }
        }
        //
        // 只要首页查询成功，page就已经是0了。
        if ($scope.page >= 0 && $scope.page < $scope.totalPages - 1) {
          $scope.loadMoreFailed = false;
          $scope.searchBooksOfNextPageBycategory();
        } else {
          console.warn("当前已经是最后一页，不需要加载下一页。");
        }
      };
      // 查询图书
      $scope.searchBooksOfNextPageBycategory = function () {
        if ($scope.page == $scope.totalPages - 1 && $scope.totalPages >= 0) {
          console.warn("分类查询图书列表，已经是最后一页，不能再往后查询了。");
          return;
        }
        if ($scope.isQueryAuto) {
          uiService.ionicLoadingShow({ templateTitle: "正在查询，请稍候..." });
        }
        apiService.searchBooksBycategory(
          {
            page: $scope.page + 1,
            size: $scope.size,
            categoryCode: $scope.categoryCode,
            accountId: $scope.account,
          },
          {
            onSuccess: function (data, status, config, headers) {
              console.log("searchBooksBycategory. onSuccess.");
              // 空数据，应该认为操作失败。当前场景应该直接返回。
              if (_.isEmpty(_.trim(data))) {
                // 判断当前是下拉操作还是上拉操作
                if ($scope.page == -1) {
                  // 下拉：首页
                  $scope.refreshFailed = true;
                } else {
                  // 上拉：原始页码大于等于0
                  $scope.loadMoreFailed = true;
                }
                // 上传日志
                var error_desciption =
                  "根据分类查询图书列表, 后台返回的数据为空空内容。";
                console.warn(
                  "searchBooksBycategory. onSuccess. but data is empty."
                );
                $scope.submitErrorLog(data, error_desciption);
                return;
              }

              // 处理一个异常情况：当前页期望图书本书和实际返回值不一致，上传日志。
              var error_desciption = undefined;
              try {
                //
                if (!data.data) {
                  error_desciption =
                    "后台返回的图书列表数据为空空内容（不是长度为0，是数据为空。）";
                } else {
                  if ($scope + 1 < $scope.totalPages - 1) {
                    if (data.data.length != $scope.size) {
                      error_desciption =
                        "后台返回的图书列表数据的长度与实际值不一样，期望返回图书本数为" +
                        $scope.size +
                        "，实际返回本书为" +
                        data.data.length;
                    }
                  } else if ($scope + 1 < $scope.totalPages - 1) {
                    if (
                      data.data.length !=
                      $scope.elementsCount - $scope.books.length
                    ) {
                      error_desciption =
                        "后台返回的图书列表数据的长度与实际值不一样，期望返回图书本数为" +
                        ($scope.elementsCount - $scope.books.length) +
                        "，实际返回本书为" +
                        data.data.length;
                    }
                  }
                }
              } catch (err) {
              } finally {
                if (error_desciption) {
                  // 判断当前是下拉操作还是上拉操作
                  if ($scope.page == -1) {
                    // 下拉：首页
                    $scope.refreshFailed = true;
                  } else {
                    // 上拉：原始页码大于等于0
                    $scope.loadMoreFailed = true;
                  }
                  log.warn(error_desciption);
                  $scope.submitErrorLog(data, error_desciption);
                  return;
                }
              }

              // 判断当前是下拉操作还是上拉操作
              if ($scope.page == -1) {
                // 下拉：首页
                $scope.refreshFailed = false;
              } else {
                // 上拉：原始页码大于等于0
                $scope.loadMoreFailed = false;
              }
              // 查询成功，赋值数据
              $scope.elementsCount = data.elementsCount;
              $scope.totalPages = data.totalPages;
              $scope.page = $scope.page + 1;
              $scope.books = $scope.books.concat(data.data);
            },
            onError: function (data, status, config, headers) {
              console.warn("searchBooksBycategory. onError.");
              // 判断当前是下拉操作还是上拉操作
              if ($scope.page == -1) {
                // 下拉：首页
                $scope.refreshFailed = true;
              } else {
                // 上拉：原始页码大于等于0
                $scope.loadMoreFailed = true;
              }
              // 上传日志
              $scope.submitErrorLog(data, null);
            },
            onFinally: function () {
              // 隐藏进度条
              if ($scope.isQueryAuto == true) {
                $scope.$broadcast("scroll.refreshComplete");
              } else {
                $scope.$broadcast("scroll.infiniteScrollComplete");
              }
              if ($scope.isQueryAuto) {
                $scope.isQueryAuto = false;
                $ionicLoading.hide();
              } else {
                if ($scope.page == -1) {
                  // 首页查询失败
                  $scope.$broadcast("scroll.refreshComplete");
                } else if ($scope.page == 0 && !$scope.refreshFailed) {
                  // 首页查询成功
                  $scope.$broadcast("scroll.refreshComplete");
                } else if ($scope.page == 0 && $scope.loadMoreFailed) {
                  // 第一次更多查询失败
                  $scope.$broadcast("scroll.infiniteScrollComplete");
                } else if ($scope.page > 0) {
                  // 第n>1次查询更多
                  $scope.$broadcast("scroll.infiniteScrollComplete");
                }
              }
            },
          }
        );
      };

      // ----------------------------------------------------------------------------------------------------------------
      $scope.submitErrorLog = function (server_data, desciption) {
        desciption =
          "根据图书分类查询图书列表遇到异常。错误信息是：" +
          desciption +
          "。当前参数是：category=" +
          JSON.stringify($scope.category) +
          "，totalPages=" +
          $scope.totalPages +
          "，elementsCount=" +
          $scope.elementsCount +
          "，size=" +
          $scope.size +
          "，期望page=" +
          ($scope.page + 1) +
          "。";
        var log = appLogService.getBaseLog();
        log.level = "warn";
        log.category = "module";
        log.module = "search_book";
        log.error = {
          desciption: desciption,
          category: angular.toJson($scope.category),
          totalPages: $scope.totalPages,
          elementsCount: angular.toJson($scope.elementsCount),
          size: $scope.size,
          search_page: $scope.page + 1,
          actual_data_from_server: angular.toJson(server_data),
        };
        appLogService.uploadLog(log, {
          onSuccess: function (data, status, config, headers) {
            console.log("提交图书分类查询异常日志-->成功");
          },
          onError: function (data, status, config, headers) {
            console.error("提交图书分类查询异常日志-->失败");
          },
          onFinally: function () {
            //
          },
        });
      };

      // ----------------------------------------------------------------------------------------------------------------
      $scope.goDetail = function (book) {
        $state.go("tab.book-detail", { book: book });
      };

      // ----------------------------------------------------------------------------------------------------------------
      // 主动搜索一次
      $scope.isQueryAuto = true;
      $scope.refresh();
      $ionicTabsDelegate.showBar(false);
    }
  );
