angular
  .module("starter.controllers")

  .controller(
    "BookroomLocationCtrl",
    function (
      $http,
      $scope,
      $ionicTabsDelegate,
      $ionicActionSheet,
      ngCordovaService
    ) {
      $ionicTabsDelegate.showBar(false);
      $scope.json_data = [];
      $scope.bookname = [];
      $scope.pointArray = [];
      $scope.map = new AMap.Map("container", {
        resizeEnable: true, //是否监控地图容器尺寸变化
        zoom: 11, //初始化地图层级
        center: [116.359444, 39.75515], //初始化地图中心点
      });

      $scope.initLibraryLocation = function () {
        $http
          .get(
            "http://bookroom_api.leituzhiyue.com:8081/management/smart_library/smart_library_rooms?user_name=daxing_location"
          )
          .success(function (resp) {
            if (parseInt(resp.result_code) == 1000) {
              var data = resp.data;

              for (var i = 0; i < data.length; i++) {
                $scope.marker = new AMap.Marker({
                  position: [
                    parseFloat(data[i].longitude),
                    parseFloat(data[i].latitude),
                  ],
                  title: data[i].smart_library_name,
                  map: $scope.map,
                });
                $scope.marker.content = data[i].smart_library_name;

                $scope.marker.on("click", markerClick);

                $scope.marker.setLabel({
                  offset: new AMap.Pixel(-5, -20), //设置文本标注偏移量
                  content:
                    "<div class='info'>" +
                    data[i].smart_library_name +
                    "</div>", //设置文本标注内容
                  direction: "right", //设置文本标注方位
                });
              }
            } else {
              ngCordovaService.showToast(resp.data);
            }
          })
          .error(function (error) {
            ngCordovaService.showToast("服务器连接失败，请稍后尝试");
          });
      };

      $scope.initLibraryLocation();

      function markerClick(e) {
        var content = e.target.content;
        var p = e.target.getPosition();
        $scope.showAction(content, p);
      }
      $scope.showAction = function (content, postion) {
        var lng = postion.lng;
        var lat = postion.lat;
        $ionicActionSheet.show({
          buttons: [{ text: "高德地图" }, { text: "腾讯地图" }],
          titleText: "地图导航",
          cancelText: "取消",
          cancel: function () {
            // add cancel code..
          },
          buttonClicked: function (index) {
            let schemeIntent;
            let url;
            if (index == 0) {
              //高德地图

              if (ionic.Platform.isIPad() || ionic.Platform.isIOS()) {
                schemeIntent = "iosamap://";
                url =
                  "iosamap://path?sourceApplication=com.leituzhiyue.daxinglibrary&dlat=" +
                  lat +
                  "&dlon=" +
                  lng +
                  "&dname=" +
                  content +
                  "&dev=0&t=0";
                appAvailability.check(
                  schemeIntent,
                  $scope.hasIosPackage(url),
                  $scope.notIosPackage()
                ); //IOS
              } else {
                schemeIntent = "com.autonavi.minimap";
                url =
                  "amapuri://route/plan/?dlat=" +
                  lat +
                  "&dlon=" +
                  lng +
                  "&dname=" +
                  content +
                  "&dev=0&t=0";
                appAvailability.check(
                  schemeIntent,
                  $scope.hasAndroidPackage(schemeIntent, url),
                  $scope.notAndroidPackage()
                );
              }
            } else {
              //腾讯地图
              url =
                "qqmap://map/routeplan?type=drive&fromcoord=CurrentLocation&to=" +
                content +
                "&tocoord=" +
                lat +
                "," +
                lng +
                "&referer=GBMBZ-SZ6CX-Z2Z4C-ZK7UV-F3K6S-FUBEQ";
              if (ionic.Platform.isIPad() || ionic.Platform.isIOS()) {
                schemeIntent = "qqmap://";
                appAvailability.check(
                  schemeIntent,
                  $scope.hasIosPackage(url),
                  $scope.notIosPackage()
                );
              } else {
                schemeIntent = "com.tencent.map";
                appAvailability.check(
                  schemeIntent,
                  $scope.hasAndroidPackage(schemeIntent, url),
                  $scope.notAndroidPackage()
                );
              }
            }
            return true;
          },
        });
      };

      $scope.hasAndroidPackage = function (schemeIntent, url) {
        var sApp = startApp.set(
          {
            //跳转对应APP
            action: "ACTION_VIEW",
            category: "CATEGORY_DEFAULT",
            type: "text/css",
            package: schemeIntent,
            uri: url, //路径规划然后导航的
            flags: ["FLAG_ACTIVITY_CLEAR_TOP", "FLAG_ACTIVITY_CLEAR_TASK"],
            intentstart: "startActivity",
          },
          {
            /* extras */
            EXTRA_STREAM: "extraValue1",
            extraKey2: "extraValue2",
          }
        );
        sApp.start(
          function () {
            //跳转成功
            console.log("地图app加载成功");
          },
          function (error) {
            //失败

            ngCordovaService.showToast("请更换地图APP或下载该地图APP");

            //alert('请下载该地图APP');
          }
        );
      };

      $scope.notAndroidPackage = function () {
        // 不存在对应APP
        console.log("未安装app");
      };

      $scope.hasIosPackage = function (url) {
        // 存在对应APP
        var sApp = startApp.set(url);
        sApp.start(
          function () {
            console.log("地图app加载成功");
          },
          function (error) {
            ngCordovaService.showToast("请更换地图APP或下载该地图APP");
          }
        );
      };
      $scope.notIosPackage = function () {
        // 不存在对应APP
        console.log("请更换地图APP或下载该地图APP");
      };
    }
  );
